<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Util\ControladorAlerta;

use Ems\Services\ApuracaoService;

class ApuracaoController {
    /** @var ApuracaoService */
    protected $apuracaoService;

    public function __construct(ApuracaoService $apuracaoService) {
        $this->apuracaoService = $apuracaoService;
    }

    public function retrocederApuracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo = $row['NRVINCULOM'];
            $nrPeriodoApuracao = $row['NRPERIODOAPURACAO'];

            $qntDeletedRows = $this->apuracaoService->retrocederApuracao($nrVinculo, $nrPeriodoApuracao);

            $retrocederApuracao = [
                'deletedRows' => $qntDeletedRows
            ];

            $response->addDataSet(new DataSet('retrocederApuracao', $retrocederApuracao));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function apurarPeriodo(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];
            $nrPeriodoApuracao = (int) $row['NRPERIODOAPURACAO'];

            $apurado = $this->apuracaoService->apurarPeriodo($nrVinculo, $nrPeriodoApuracao);

            $dataset = [
                'apurado' => $apurado
            ];

            $response->addDataSet(new DataSet('apurarPeriodo', $dataset));
            $response->addDataSet(new DataSet('info', ControladorAlerta::getMensagensProcessoAtual()));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function apurarDias(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int) $row['NRVINCULOM'];
            $dias               =       $row['DIAS'];

            $apurado = $this->apuracaoService->apurarDias(
                $nrVinculo,
                $dias
            );

            $dataset = [
                'apurado' => $apurado
            ];

            $response->addDataSet(new DataSet('apurarDias', $dataset));
            $response->addDataSet(new DataSet('info', ControladorAlerta::getMensagensProcessoAtual()));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function verificaDiaTemResumo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $temResumo = $this->apuracaoService->verificaDiaTemResumo(
                $nrVinculo, $date
            );

            $dataset = [
                'temResumo' => $temResumo
            ];

            $response->addDataSet(new DataSet('verificaDiaTemResumo', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function apurarPeriodoFuncionario(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];
            $nrPeriodo = (int) $row['NRPERIODOAPURACAO'];

            $this->apuracaoService->apurarPeriodoFuncionario(
                $nrVinculo, $nrPeriodo
            );

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('apurarPeriodoFuncionario', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
