<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Mail\SendEmail\SendEmail;
use Util\Repositories;
use Util\Environment;
use Util\DateUtil;
use Util\Messages;
use Util\EmailMessages;
use Util\ControladorAlerta;

use Ems\Services\VinculoService;
use Ems\Services\ParceiroNegocioService;
use Ems\Services\OperadorService;
use Ems\Services\EstruturaService;
use Ems\Services\ParamOrgService;
use Ems\Services\SolicitacaoService;
use Ems\Util\Utils;

use Ems\Repository\MarcacaoRepository;

class EmailService {
    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;

    public function __construct(EntityManager $entityManager, Environment $environment, VinculoService $vinculoService, ParceiroNegocioService $parceiroNegocioService, OperadorService $operadorService, EstruturaService $estruturaService, SolicitacaoService $solicitacaoService) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->vinculoService           = $vinculoService;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->operadorService          = $operadorService;
        $this->estruturaService         = $estruturaService;
        $this->solicitacaoService       = $solicitacaoService;
    }
    
    public const REMETENTE_PG = 'Teknisa Portal do Gestor';
    public const REMETENTE_PF = 'Teknisa Portal do Funcionário';
    public const REMETENTE_DEFAULT = 'Teknisa Portais';
    public const TEMPLATE_DEFAULT = 'TEKNISA_TEMPLATE';
    public const ADDRESSCC_DEFAULT = [];
    public const ADDRESSBCC_DEFAULT = [];
    public const VALIDAEMAILTABELAOPER = true;
    public const NAOVALIDAEMAILTABELAOPER = false;
    public const EMPTY_FIELD_DEFAULT_VALUE = 'N/A';
    
    /**
     * Rotina para envio de emails de notificação de ocorrência para os parceiros de negócio associados ao perfil de acesso da estrutura
     * 
     * @param string $cdoperador
     * @param Array  $ocorrencia
     *      array(
     *          IDPROGOCORCONFIRMADA
     *          DTINICIOPROGOCOR
     *          DTFIMPROGOCOR
     *          DSMOTIVOOCORFREQ
     *          DSOBSERVACAO
     *          DATA_ALTERACAOSTATUS
     *          HORA_ALTERACAOSTATUS
     *          NOME_VINCULO
     *      )
     * @param string $baseContent
     * @param string $baseSubject
     * @param string $attachment            Default: null
     * @param bool   $validaEmailTabelaOper Default: self::VALIDAEMAILTABELAOPER
     * @param string $nmremetente           Default: self::REMETENTE_DEFAULT
     * @param string $templateName          Default: self::TEMPLATE_DEFAULT 
     */
    public function sendEmailNotificacaoOcorrencia(
        string $cdoperador,
        Array  $ocorrencia,
        string $baseContent,
        string $baseSubject,
        string $attachment = null,
        bool   $validaEmailTabelaOper       = self::VALIDAEMAILTABELAOPER,
        string $nmremetente                 = self::REMETENTE_DEFAULT,
        string $templateName                = self::TEMPLATE_DEFAULT,
        string $perfilAcessoDestinatarios   = ParamOrgService::PERFIL_ACESS_ADM_DP_RH
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $parceirosNegocio = $this->parceiroNegocioService->getParceirosNegocioPerfilAcesso($perfilAcessoDestinatarios);
        
        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $cdoperador));
        if (!empty($operador)) {
            $parceiroNegocioOperador = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $operador->getNrparcnegocio()));
            $nmoperadornotificacao = $parceiroNegocioOperador->getNmprincipalparc();
        } else {
            throw new \Exception(Messages::MAIL_OPERADOR_NOT_FOUND);
        }
        
        $emailsAEnviar = array();
        
        // Percorre os parceiros montando os emails
        foreach ($parceirosNegocio as $parcnegocio) {
            
            $nrparcnegocio = $parcnegocio['NRPARCNEGOCIO'];
            $email = $this->parceiroNegocioService->getEmailParcNegocio($nrparcnegocio);
            
            if (empty($email)) {
                continue;
            }
            
            $nmdestinatario = $parcnegocio['NMPRINCIPALPARC'];
            $nomevinculoocorrencia = $ocorrencia['NOME_VINCULO'];
            
            $vinculom = $this->vinculoService->getVinculoMestreHistoricoByNrparcnegocio($nrparcnegocio);
            $nrestruturam = $vinculom['NRESTRUTLEGAL'];
            
            
            $contentReplace = array(
                '@NOME_GESTOR@'             => $nmoperadornotificacao,
                '@NOME_VINCULO@'            => $nomevinculoocorrencia,
                '@IDPROGOCORCONFIRMADA@'    => $ocorrencia['IDPROGOCORCONFIRMADA'],
                '@DTINICIOPROGOCOR@'        => $ocorrencia['DTINICIOPROGOCOR'],
                '@DTFIMPROGOCOR@'           => !empty($ocorrencia['DTFIMPROGOCOR'])         ? $ocorrencia['DTFIMPROGOCOR']          : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DSMOTIVOOCORFREQ@'        => !empty($ocorrencia['DSMOTIVOOCORFREQ'])      ? $ocorrencia['DSMOTIVOOCORFREQ']       : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DSOBSERVACAO@'            => !empty($ocorrencia['DSOBSERVACAO'])          ? $ocorrencia['DSOBSERVACAO']           : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DATA_ALTERACAOSTATUS@'    => !empty($ocorrencia['DATA_ALTERACAOSTATUS'])  ? $ocorrencia['DATA_ALTERACAOSTATUS']   : (new \DateTime)->format(DateUtil::FORMATO_DATA),
                '@HORA_ALTERACAOSTATUS@'    => !empty($ocorrencia['HORA_ALTERACAOSTATUS'])  ? $ocorrencia['HORA_ALTERACAOSTATUS']   : (new \DateTime)->format(DateUtil::FORMATO_HORA)
            );
            
            $emailContent = strtr($baseContent, $contentReplace);
            
            
            $subjectReplace = array(
                '@NOME_VINCULO@' => $nomevinculoocorrencia
            );
            
            $emailSubject = strtr($baseSubject, $subjectReplace);
            
            $emailParams = array(
                'templateName' => $templateName,
                'email' => $email,
                'subject' => $emailSubject,
                'nmdestinatario' => $nmdestinatario,
                'nmremetente' => $nmremetente,
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'content' => $emailContent,
                'attachment' => $attachment,
                'validaEmailTabelaOper' => $validaEmailTabelaOper
            );
            
            $emailsAEnviar[] = $emailParams;
        }
        
        // Envia os emails
        $emailsEnviados = $this->sendEmails($emailsAEnviar);
        
        return $emailsEnviados;
    }
    
    /**
     * Rotina para envio de emails de notificação de programação de férias para os parceiros de negócio associados ao perfil de acesso da estrutura
     * 
     * @param string $cdoperador
     * @param Array  $programacaoFerias
     *      array(
     *          DTINIPREVISAO
     *          QTDIASPREVISAO
     *          DTFINPREVISAO
     *          QTABONOPREVISAO
     *      )
     * @param string $baseContent           Default: EmailMessages::ALTERPROGFERIAS_BODYMESSAGE
     * @param string $baseSubject           Default: EmailMessages::ALTERPROGFERIAS_SUBJECT
     * @param string $attachment            Default: null
     * @param bool   $validaEmailTabelaOper Default: self::VALIDAEMAILTABELAOPER
     * @param string $nmremetente           Default: self::REMETENTE_DEFAULT
     * @param string $templateName          Default: self::TEMPLATE_DEFAULT 
     */
    public function sendEmailNotificacaoProgramacaoFerias(
        string $cdoperador,
        Array  $programacaoFerias,
        string $baseContent                 = EmailMessages::ALTERPROGFERIAS_BODYMESSAGE,
        string $baseSubject                 = EmailMessages::ALTERPROGFERIAS_SUBJECT,
        string $attachment                  = null,
        bool   $validaEmailTabelaOper       = self::VALIDAEMAILTABELAOPER,
        string $nmremetente                 = self::REMETENTE_DEFAULT,
        string $templateName                = self::TEMPLATE_DEFAULT,
        string $perfilAcessoDestinatarios   = ParamOrgService::PERFIL_ACESS_ADM_DP_RH
    ) {
        
        // Dados que serão presentes em todos os emails
        $nrorg = $this->environment->getNrorg();
        
        $parceirosNegocio = $this->parceiroNegocioService->getParceirosNegocioPerfilAcesso($perfilAcessoDestinatarios);
        
        // Operador
        $operador = $this->getDadosOperador($cdoperador, $nrorg);
        
        // Vinculo
        if (empty($programacaoFerias['NRVINCULOM'])) {
            throw new \Exception(Messages::VINC_NOT_FOUND);
        }
        
        $vinculo = $this->getDadosVinculo($programacaoFerias['NRVINCULOM'], $nrorg);
        
        // Unidade
        $unidade = $this->getDadosEstrutura($vinculo['NRESTRUTGEREN'], $nrorg);
        
        $emailsAEnviar = array();
        
        // Percorre os parceiros montando os emails com dados especificos de cada parceiro de negócio
        foreach ($parceirosNegocio as $parcnegocio) {
            
            $nrparcnegocio = $parcnegocio['NRPARCNEGOCIO'];
            $email = $this->parceiroNegocioService->getEmailParcNegocio($nrparcnegocio);
            
            if (empty($email)) {
                continue;
            }
            
            $nmdestinatario = $parcnegocio['NMPRINCIPALPARC'];
            
            $contentReplace = array(
                '@UNIDADE@'         => $unidade['NMESTRUTURAH'],
                '@NOME_GESTOR@'     => $operador['NMOPERADOR'],
                '@EMAIL_GESTOR@'    => $operador['EMAILOPERADOR'],
                '@NOME_VINCULO@'    => $vinculo['NMVINCULOM'],
                '@DTINIPREVISAO@'   => !empty($programacaoFerias['DTINIPREVISAO'])      ? $programacaoFerias['DTINIPREVISAO']   : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@QTDIASPREVISAO@'  => !empty($programacaoFerias['QTDIASPREVISAO'])     ? $programacaoFerias['QTDIASPREVISAO']  : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DTFINPREVISAO@'   => !empty($programacaoFerias['DTFINPREVISAO'])      ? $programacaoFerias['DTFINPREVISAO']   : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@QTABONOPREVISAO@' => !empty($programacaoFerias['QTABONOPREVISAO'])    ? $programacaoFerias['QTABONOPREVISAO'] : self::EMPTY_FIELD_DEFAULT_VALUE
            );
            
            $emailContent = strtr($baseContent, $contentReplace);
            
            
            $subjectReplace = array(
                '@NOME_VINCULO@' => $vinculo['NMVINCULOM']
            );
            
            $emailSubject = strtr($baseSubject, $subjectReplace);
            
            
            $emailParams = array(
                'templateName' => $templateName,
                'email' => $email,
                'subject' => $emailSubject,
                'nmdestinatario' => $nmdestinatario,
                'nmremetente' => $nmremetente,
                'nrorg' => $nrorg,
                'nrestruturam' => $vinculo['NRESTRUTLEGAL'],
                'content' => $emailContent,
                'attachment' => $attachment,
                'validaEmailTabelaOper' => $validaEmailTabelaOper
            );
            
            $emailsAEnviar[] = $emailParams;
        }
        
        // Envia os emails
        $emailsEnviados = $this->sendEmails($emailsAEnviar);
        
        return $emailsEnviados;
    }
    
    /**
     * Rotina para envio de emails de notificação de programação de férias para os parceiros de negócio associados ao perfil de acesso da estrutura
     * 
     * @param Array  $params
     *      array(
     *          NRVINCULOM_LIST
     *          NRESTRUTLEGAL_LIST
     *          NRESTRUTGEREN_LIST
     *          NRTIPOVINCULOM_LIST
     *          NRCOLETOR_LIST
     *          DTINICIOVERIFICACAO
     *          DTFINALVERIFICACAO
     *      )
     * @param string $baseContent           Default: EmailMessages::ALERTA_MARCACACOES_BODYMESSAGE
     * @param string $baseSubject           Default: EmailMessages::ALERTA_MARCACACOES_SUBJECT
     * @param string $attachment            Default: null
     * @param bool   $validaEmailTabelaOper Default: self::VALIDAEMAILTABELAOPER
     * @param string $nmremetente           Default: self::REMETENTE_DEFAULT
     * @param string $templateName          Default: self::TEMPLATE_DEFAULT 
     */
    public function sendEmailAlertaRegistrosFaltantesUltimaSemana(
        array  $params                      = [],
        string $baseContent                 = EmailMessages::ALERTA_MARCACACOES_BODYMESSAGE,
        string $baseSubject                 = EmailMessages::ALERTA_MARCACACOES_SUBJECT,
        string $attachment                  = null,
        bool   $validaEmailTabelaOper       = self::VALIDAEMAILTABELAOPER,
        string $nmremetente                 = self::REMETENTE_DEFAULT,
        string $templateName                = self::TEMPLATE_DEFAULT
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $nrVinculomList = $params['NRVINCULOM_LIST'] ?? null;
        $nrEstrutlegalList = $params['NRESTRUTLEGAL_LIST'] ?? null;
        $nrEstrutgerenList = $params['NRESTRUTGEREN_LIST'] ?? null;
        $nrTipovinculomList = $params['NRTIPOVINCULOM_LIST'] ?? null;
        $nrColetorList = $params['NRCOLETOR_LIST'] ?? null;
        $dtInicioVerificacao = $params['DTINICIOVERIFICACAO'] ?? null;
        $dtFinalVerificacao = $params['DTFINALVERIFICACAO'] ?? null;
        
        $vinculosConteudoASerEnviado = Utils::getInstanceByClassName(MarcacaoRepository::class)->getAlertaRegistrosFaltantesUltimaSemana(
            $nrVinculomList,
            $nrEstrutlegalList,
            $nrEstrutgerenList,
            $nrTipovinculomList,
            $nrColetorList,
            $dtInicioVerificacao,
            $dtFinalVerificacao
        );
        
        $emailsAEnviar = array();
        
        foreach ($vinculosConteudoASerEnviado as $vinculoConteudoASerEnviado) {
            $nmdestinatario = $vinculoConteudoASerEnviado['NMVINCULOM'];
            $email = $vinculoConteudoASerEnviado['EMAIL'];
            
            /* Itens */
            
            $mensagemBaseItens = EmailMessages::ALERTA_MARCACACOES_ITEM_CONTENT;
            $datas = explode(',', $vinculoConteudoASerEnviado['DATAS'] ?? '');
            $mensagemItens = '';
            foreach ($datas as $data) {
                $itensReplace = array(
                    '@DATA@' => $data,
                );
                $mensagemItens .= strtr($mensagemBaseItens, $itensReplace);
            }
            
            $contentReplace = array(
                '@ITENS_ALERTA@'   => $mensagemItens,
            );
            
            $emailContent = strtr($baseContent, $contentReplace);
            
            $emailParams = array(
                'templateName' => $templateName,
                'email' => $email,
                'subject' => $baseSubject,
                'nmdestinatario' => $nmdestinatario,
                'nmremetente' => $nmremetente,
                'nrorg' => $nrorg,
                'nrestruturam' => $vinculoConteudoASerEnviado['NRESTRUTLEGAL'],
                'content' => $emailContent,
                'attachment' => $attachment,
                'validaEmailTabelaOper' => $validaEmailTabelaOper
            );
            
            $emailsAEnviar[] = $emailParams;
        }
        
        // Envia os emails
        $emailsEnviados = $this->sendEmails($emailsAEnviar);
        
        return $emailsEnviados;
    }
    
    /**
     * Rotina de envio individual de email
     * 
     * @param string $email
     * @param string $subject
     * @param string $nmdestinatario
     * @param string $content
     * @param string $nrestruturam
     * @param string $nmremetente             Default: self::REMETENTE_DEFAULT
     * @param string|array $attachment        Default: null
     * @param bool   $validaEmailTabelaOper   Default: self::VALIDAEMAILTABELAOPER
     * @param string $templateName            Default: self::TEMPLATE_DEFAULT
     */
    public function sendEmail(
        ?string $email,
        string $subject,
        ?string $nmdestinatario,
        string $content,
        string $nrestruturam,
        string $nmremetente             = self::REMETENTE_DEFAULT,
        $attachment        = null,
        bool $validaEmailTabelaOper     = self::VALIDAEMAILTABELAOPER,
        string $templateName            = self::TEMPLATE_DEFAULT,
        array $addressCC                = self::ADDRESSCC_DEFAULT,
        array $addressBCC               = self::ADDRESSBCC_DEFAULT
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $htmlBodyContent = array(
            "SUBJECT"                 => $subject,
            "STRING_SUPERIOR_DIREITA" => null,
            "CONTENT_BODY"            => $content  
        );
        
        return SendEmail::sendEmail(
            $templateName, $email, $subject, $nmdestinatario,
            $nmremetente, $nrorg, $nrestruturam, $htmlBodyContent,
            $attachment, $validaEmailTabelaOper, $addressCC, $addressBCC
        );
    }
    
    /**
     * Rotina de envio de vários emails simultâneamente
     * 
     * @param Array EmailsParams $emailsParams
     * 
     * EmailsParams:
     *  string $email
     *  string $subject
     *  string $nmdestinatario
     *  string $content
     *  string $nrestruturam
     *  string $nmremetente             Default: self::REMETENTE_DEFAULT
     *  string $attachment              Default: null
     *  bool   $validaEmailTabelaOper   Default: self::VALIDAEMAILTABELAOPER
     *  string $templateName            Default: self::TEMPLATE_DEFAULT
     */
    public function sendEmails(Array $emailsParams) {
        foreach ($emailsParams as $emailParams) {
            
            // Default parameters
            $nmremetente            = !empty($emailParams['nmremetente'])           ? $emailParams['nmremetente']           : null;
            $attachment             = !empty($emailParams['attachment'])            ? $emailParams['attachment']            : null;
            $validaEmailTabelaOper  = !empty($emailParams['validaEmailTabelaOper']) ? $emailParams['validaEmailTabelaOper'] : null;
            $templateName           = !empty($emailParams['templateName'])          ? $emailParams['templateName']          : null;
            
            try {
                $emailEnviado = $this->sendEmail(
                    $emailParams['email'],
                    $emailParams['subject'],
                    $emailParams['nmdestinatario'],
                    $emailParams['content'],
                    $emailParams['nrestruturam'],
                    $emailParams['nmremetente'],
                    $emailParams['attachment'],
                    $emailParams['validaEmailTabelaOper'],
                    $emailParams['templateName']
                );
                
                if (!$emailEnviado) {
                    throw new \Exception(Messages::MAIL_NOT_SENT);
                }
                ControladorAlerta::adicionaMensagemProcessoAtual("Foi realizado com sucesso o envio para o endereço ${emailParams['email']}", ControladorAlerta::TIPO_SUCCESS);
            } catch (\Exception $e) {
                $emailsNaoEnviados[] = array('email' => $emailParams['email'], 'erro' => $e->getMessage());
                ControladorAlerta::adicionaMensagemProcessoAtual("O envio para o endereço ${emailParams['email']} não foi realizado. " . $e->getMessage());
            }
        }
        
        // Emails enviados com sucesso
        if (empty($emailsNaoEnviados)) {
            return true;
        }
        
        // Retorna emails nao enviados
        return $emailsNaoEnviados;
    }
    
    public function agrupaMensagensEmail($mensagensEmail) {
        if (empty($mensagensEmail)) {
            return false;
        }
        
        $mensagem = Messages::MAILS_NOT_SENT . '<br>';
        foreach ($mensagensEmail as $mensagemEmail) {
            $mensagem .= 'Email: ' . $mensagemEmail['email'] . '. Erro: ' . $mensagemEmail['erro'] . '<br>';
        }
        
        return $mensagem;
    }
    
    /**
     *  Função que retorna dados úteis do operador para as rotinas de EMAIL. 
     * 
     *  @param string $cdoperador
     * 
     *  @return Array $dadosOperador
     *      'NMOPERADOR'    => $nmoperadornotificacao,
     *      'EMAILOPEADOR'  => $emailoperadornotificacao
     * 
     *  @throws Exception
     */
    public function getDadosOperador($cdoperador, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosOperador = $this->operadorService->getDadosOperador($cdoperador);
        
        return $dadosOperador;
    }
    
    /**
     *  Função que retorna dados úteis do vínculo para as rotinas de EMAIL. 
     * 
     *  @param string $nrvinculom
     * 
     *  @return Array $dadosVinculo
     *      'NMVINCULOM'    => $nmvinculom
     * 
     *  @throws Exception
     */
    public function getDadosVinculo($nrvinculom, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosVinculo = $this->vinculoService->getDadosVinculo($nrvinculom, $nrorg);
        
        return $dadosVinculo;
    }
    
    /**
     *  Função que retorna dados úteis da estrutura para as rotinas de EMAIL. 
     * 
     *  @param string $nrvinculom
     * 
     *  @return Array $dadosVinculo
     *      'NMVINCULOM'    => $nmvinculom
     * 
     *  @throws Exception
     */
    public function getDadosEstrutura($nrestruturam, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosEstrutura = $this->estruturaService->getDadosEstrutura($nrestruturam, $nrorg);
        
        return $dadosEstrutura;
    }
    
    
    /**
    *   Função usada para enviar email de notificação de solicitação da tela de Minha história 
    *
    * 
    **/
    public function notificaMinhaHistoria(
                $nrvinculom,
        ?int    $nrSolcitacao,
        ?int    $nrTipoSolicitacao,
        array   $onlyInEmailItems            = [],
        $attachment                          = null,
        string  $baseContent                 = EmailMessages::SOLICITACAO_BODYMESSAGE,
        string  $baseSubject                 = EmailMessages::SOLICITACAO_SUBJECT,
        bool    $validaEmailTabelaOper       = self::NAOVALIDAEMAILTABELAOPER,
        string  $nmremetente                 = self::REMETENTE_DEFAULT,
        string  $templateName                = self::TEMPLATE_DEFAULT
    ){
        /* Condicao temporaria */
        if ($this->environment->getNrorg() == 3260 || $this->environment->getNrorg() == 1410) {
            return;
        }

        /* Dados gerais do email */
        
        $nrorg = $this->environment->getNrorg();
        $vinculo = $this->getDadosVinculo($nrvinculom);
        $unidade = $this->getDadosEstrutura($vinculo['NRESTRUTGEREN'], $nrorg);
        
        $nmGerem = $unidade['NMESTRUTURAM'];
        $nrEstrutLegal = $vinculo['NRESTRUTLEGAL']; 
        $nmVinculo = $vinculo['NMVINCULOM'];
        $nmTipoVinculo = $vinculo['NMTIPOVINCULOH'];

        $operadores = $this->operadorService->getOperadoresDp();
        if (empty($operadores)) {
            return;
        }
        
        $dados = $this->estruturaService->retornaDadosResponsavelInformal($nrorg, $unidade['NRESTRUTURAM'], $this->environment->getDtmescomptencDiaCorrente());
        
        // Unidade
        $estrutGerenReplace = array(
            '@NOME_UNIDADE@' => $nmGerem,
        );
        
        $emailResp = strtr(EmailMessages::EMAIL_RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
        $nmdestinatarioResp = strtr(EmailMessages::RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
        if (!empty($dados)) {
            $emailResp = $dados['CDCOMUNICAPARC'];
            $nmdestinatarioResp = $dados['NMPESSOA'];
        }
        
        /* Itens da solicitacao */
        
        // Busca itens da solicitação. Merge dos itens a serem incluidos no email somente
        $itensSolicitacao = $this->solicitacaoService->getItemSolicitacao($nrSolcitacao);
        $itensSolicitacao = array_merge($onlyInEmailItems, $itensSolicitacao);
        
        /* Solicitacao */
        
        $solicitacao = $this->getSolicitacao($nrSolcitacao, $nrTipoSolicitacao);

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $solicitacao['CDOPERINCLUSAOSOLI']));
        $nmOperador = '';
        if($operador) {
            $nmOperador = $operador->getNmoperador();
        }
        
        /* Preenchimento dos templates */
        
        // Subject
        $subjectReplace = array(
            '@NOME_VINCULO@' => $nmVinculo,
            '@NUMERO_SOLICITACAO@' => $solicitacao['NRSOLICITACAO'],
        );
            
        $emailSubject = strtr($baseSubject, $subjectReplace);
        
        // Itens
        $mensagemBaseItens = EmailMessages::SOLICITACAO_ITEM_CONTENT;
        $mensagemItens = '';
        foreach ($itensSolicitacao as $key => $item) {
            $itensReplace = array(
                '@ITEM_LABEL@' => $item['DSLABELITEMSOLI'],
                '@ITEM_VALOR@' => $item['DSVALORITEMSOLI'],
            );
                
            $mensagemItens .= strtr($mensagemBaseItens, $itensReplace);
        }
        
        $contentReplace = array(
            '@TIPO_SOLICITACAO@'    => $solicitacao['NMTIPOSOLICITACAO'],
            '@NOME_UNIDADE@'        => $nmGerem,
            '@NOME_RESPONSAVEL@'    => $nmdestinatarioResp,
            '@EMAIL_RESPONSAVEL@'   => $emailResp,
            '@NOME_VINCULO@'        => $nmVinculo,
            '@TIPO_VINCULO@'        => $nmTipoVinculo,
            '@NOME_SOLICITANTE'     => $nmOperador,
            '@SOLI_OBSERVACAO'      => $solicitacao['DSOBSSOLICITACAO'],
            '@NUMERO_SOLICITACAO@'  => $solicitacao['NRSOLICITACAO'],
            '@ITENS_SOLICITACAO@'   => $mensagemItens,
            '@OBSERVACOES_FINAIS@'  => $solicitacao['OBSERVACOES_FINAIS'],
        );
        
        $emailContent = strtr($baseContent, $contentReplace);
        
        /* Envio dos emails */

        $emailsBCC = array_map(function ($item) { return $item['DSEMAILOPER']; }, $operadores);

        $emailEnviado = $this->sendEmail(
            null, $emailSubject, null, $emailContent,
            $nrEstrutLegal, $nmremetente, $attachment, $validaEmailTabelaOper,
            $templateName, self::ADDRESSCC_DEFAULT, $emailsBCC
        );
    }
    
    public function notificaProximosAprovadoresSolicitacao(int $nrSolcitacao): bool {

        /* Condicao temporaria */
        if ($this->environment->getNrorg() == 3260 || $this->environment->getNrorg() == 1410) {
            return true;
        }

        $infoFase = $this->solicitacaoService->getInfoFase($nrSolcitacao);

        if (empty($infoFase) || $infoFase['EXISTE_FASE_SEGUINTE'] == 'N') {
            return false;
        }

        $emailsBCC = $infoFase['DSEMAILSOPER_FASE_SEGUINTE'] ?? '';
        $emailsBCC = array_unique(explode(',', $emailsBCC));

        if (empty($emailsBCC)) {
            return false;
        }

        /* Dados gerais do email */

        $nrorg = $this->environment->getNrorg();
        $vinculo = $this->getDadosVinculo($infoFase['NRVINCULOM']);
        $unidade = $this->getDadosEstrutura($vinculo['NRESTRUTGEREN'], $nrorg);
        
        $nmGerem = $unidade['NMESTRUTURAM'];
        $nmTipoVinculo = $vinculo['NMTIPOVINCULOH'];
        $linkPGSolicitacao = 'https://portalgestor.teknisa.com/#/solicitacao?' . 'NRSOLICITACAO=' . $infoFase['NRSOLICITACAO'] . '&NRESTRUTURAM=' . $vinculo['NRESTRUTGEREN'];
        
        $dados = $this->estruturaService->retornaDadosResponsavelInformal($nrorg, $unidade['NRESTRUTURAM'], $this->environment->getDtmescomptencDiaCorrente());
        
        // Unidade
        $estrutGerenReplace = array(
            '@NOME_UNIDADE@' => $nmGerem,
        );
        
        $emailResp = strtr(EmailMessages::EMAIL_RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
        $nmdestinatarioResp = strtr(EmailMessages::RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
        if (!empty($dados)) {
            $emailResp = $dados['CDCOMUNICAPARC'];
            $nmdestinatarioResp = $dados['NMPESSOA'];
        }
        
        /* Itens da solicitacao */
        
        $itensSolicitacao = $this->solicitacaoService->getItemSolicitacao($nrSolcitacao);
        
        /* Solicitacao */
        
        $solicitacao = $this->getSolicitacao($nrSolcitacao, $infoFase['NRTIPOSOLICITACAO']);

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $solicitacao['CDOPERINCLUSAOSOLI']));
        $nmOperador = '';
        if($operador) {
            $nmOperador = $operador->getNmoperador();
        }

        /* Subject */

        $subjectReplace = array(
        '@NOME_VINCULO@' => $infoFase['NMVINCULOM'],
        '@NUMERO_SOLICITACAO@' => $infoFase['NRSOLICITACAO'],
        );
            
        $emailSubject = strtr(EmailMessages::SOLICITACAO_NOTIF_APROV_SUBJECT, $subjectReplace);

        // Itens
        $mensagemBaseItens = EmailMessages::SOLICITACAO_ITEM_CONTENT;
        $mensagemItens = '';
        foreach ($itensSolicitacao as $key => $item) {
            $itensReplace = array(
                '@ITEM_LABEL@' => $item['DSLABELITEMSOLI'],
                '@ITEM_VALOR@' => $item['DSVALORITEMSOLI'],
            );
                
            $mensagemItens .= strtr($mensagemBaseItens, $itensReplace);
        }
    
        /* Content */

        $contentReplace = array(
            '@TIPO_SOLICITACAO@'    => $infoFase['NMTIPOSOLICITACAO'],
            '@NOME_VINCULO@'        => $infoFase['NMVINCULOM'],
            '@NUMERO_SOLICITACAO@'  => $infoFase['NRSOLICITACAO'],
            '@NOME_UNIDADE@'        => $nmGerem,
            '@NOME_RESPONSAVEL@'    => $nmdestinatarioResp,
            '@EMAIL_RESPONSAVEL@'   => $emailResp,
            '@TIPO_VINCULO@'        => $nmTipoVinculo,
            '@NOME_SOLICITANTE'     => $nmOperador,
            '@SOLI_OBSERVACAO'      => $solicitacao['DSOBSSOLICITACAO'],
            '@ITENS_SOLICITACAO@'   => $mensagemItens,
            '@OBSERVACOES_FINAIS@'  => $solicitacao['OBSERVACOES_FINAIS'],
            '@LINK_PG_SOLICITACAO@' => $linkPGSolicitacao,
        );
        
        $emailContent = strtr(EmailMessages::SOLICITACAO_NOTIF_APROV_BODYMESSAGE, $contentReplace);

        /* Detalhes gerais do e-mail */
        
        $vinculo = $this->getDadosVinculo($infoFase['NRVINCULOM']);
        $nrEstrutLegal = $vinculo['NRESTRUTLEGAL']; 

        return $this->sendEmail(
            null, $emailSubject, null, $emailContent,
            $nrEstrutLegal, self::REMETENTE_DEFAULT, null, false,
            self::TEMPLATE_DEFAULT, self::ADDRESSCC_DEFAULT, $emailsBCC
        );
    }

   /**
    *   Função usada para enviar email de notificação de solicitação da tela de Minha história 
    *
    * 
    **/
    public function notificaBeneficio(
                $nrvinculom,
        ?int    $nrSolcitacao,
        array    $mensagens,
        array   $onlyInEmailItems            = [],
        string  $attachment                  = null,
        string  $baseContent                 = EmailMessages::SOLICITACAO_BODYMESSAGE,
        string  $baseSubject                 = EmailMessages::SOLICITACAO_SUBJECT,
        bool    $validaEmailTabelaOper       = self::NAOVALIDAEMAILTABELAOPER,
        string  $nmremetente                 = self::REMETENTE_DEFAULT,
        string  $templateName                = self::TEMPLATE_DEFAULT
    ){
        /* Condicao temporaria */
        if ($this->environment->getNrorg() == 3260 || $this->environment->getNrorg() == 1410) {
            return;
        }

        /* Dados gerais do email */
        
        $nrorg = 20;
        $vinculo = $this->getDadosVinculo($nrvinculom);
        $unidade = 'Luan Victor';
        
        $nmGerem = 'luan.santiago@teknisa.com';
        $nrEstrutLegal = 20; 
        $nmVinculo = 'Teste';
        $nmTipoVinculo = 'teste';
        
        // Unidade
        $estrutGerenReplace = array(
            '@NOME_UNIDADE@' => $nmGerem,
        );
        
        $emailResp = strtr(EmailMessages::EMAIL_RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
        $nmdestinatarioResp = strtr(EmailMessages::RESP_UNIDADE_NOT_FOUND, $estrutGerenReplace);
    
        // Subject
        $subjectReplace = array(
            '@NOME_VINCULO@' => $nmVinculo,
            '@NUMERO_SOLICITACAO@' => 2,
        );
            
        $emailSubject = strtr($baseSubject, $subjectReplace);
        
        // Itens
        $mensagemBaseItens = EmailMessages::BENEFICIO_EMAIL;
        $mensagemItens = '';
        foreach ($mensagens as $key => $item) {
            $itensReplace = array(
                '@ERROR_MESSAGE@' => $item
            );
                
            $mensagemItens .= strtr($mensagemBaseItens, $itensReplace);
        }
        
        $contentReplace = array(
            '@TIPO_SOLICITACAO@'    => 2,
            '@NOME_UNIDADE@'        => $nmGerem,
            '@NOME_RESPONSAVEL@'    => $nmdestinatarioResp,
            '@EMAIL_RESPONSAVEL@'   => $emailResp,
            '@NOME_VINCULO@'        => $nmVinculo,
            '@TIPO_VINCULO@'        => $nmTipoVinculo,
            '@NOME_SOLICITANTE'     => 'Luan',
            '@SOLI_OBSERVACAO'      => 'Nada',
            '@NUMERO_SOLICITACAO@'  => 2,
            '@ITENS_SOLICITACAO@'   => $mensagemItens,
            '@OBSERVACOES_FINAIS@'  => 'Teste final',
        );
        
        $emailContent = strtr($baseContent, $contentReplace);
        
        /* Envio dos emails */

        $emailsBCC[0] = 'luan.santiago@teknisa.com';

        $emailEnviado = $this->sendEmail(
            null, $emailSubject, null, $emailContent,
            $nrEstrutLegal, $nmremetente, $attachment, $validaEmailTabelaOper,
            $templateName, self::ADDRESSCC_DEFAULT, $emailsBCC
        );
    }
    
    /**
     *  Busca informacoes da solicitacao pelo NRSOLICITACAO ou NRTIPOSOLICITACAO, esse ultimo para tipos especificos nos quais so o email
     * eh enviado, sem o cadastro real da solicitacao.
     * 
     *  @param  ?int $nrSolcitacao
     *  @param  ?int $nrTipoSolicitacao
     * 
     *  @return array
     * 
     *  @throws Exception
     */
    public function getSolicitacao(?int $nrSolcitacao, ?int $nrTipoSolicitacao) {
        $observacaoFinal = MinhaHistoriaService::TIPO_SOLICITACAO_OBSERVACOES_FINAIS[$nrTipoSolicitacao] ?? '';
        
        $defaultData = [
            'OBSERVACOES_FINAIS'    => $observacaoFinal,
            'NRSOLICITACAO'         => null,
            'NMTIPOSOLICITACAO'     => null,
        ];
        
        /* Busca por tipos especificos de organizacao */
        
        if (empty($nrSolcitacao) && !empty($nrTipoSolicitacao) && SolicitacaoService::isTipoSoliOnlyEmail($nrTipoSolicitacao)) {
            $tipoSolicitacao = $this->solicitacaoService->getTiposolicitacao($nrTipoSolicitacao);
            if (!empty($tipoSolicitacao) && !empty($tipoSolicitacao[0])) {
                $defaultData['NRSOLICITACAO'] = '--';
                $defaultData['NMTIPOSOLICITACAO'] = $tipoSolicitacao[0]['NMTIPOSOLICITACAO'];
                return $defaultData;
            }
        }
        
        /* Busca convencional pela chave */
        
        $solicitacao = $this->solicitacaoService->getSolicitacao($nrSolcitacao);
        if (!empty($solicitacao) && !isset($solicitacao[0])) {
            $defaultData['NRSOLICITACAO'] = $solicitacao['NRSOLICITACAO'];
            $defaultData['NMTIPOSOLICITACAO'] = $solicitacao['NMTIPOSOLICITACAO'];
            $defaultData['DSOBSSOLICITACAO'] = $solicitacao['DSOBSSOLICITACAO'];
            $defaultData['CDOPERINCLUSAOSOLI'] = $solicitacao['CDOPERINCLUSAOSOLI'];
            return $defaultData;
        }
        
        throw new \Exception('A solicitação não foi encontrada no momento do disparo de emails.');
    }

    public function enviarEmailsOcorrenciaDocumento(
        array $operadoresDP,
        string $subject,
        string $content,
        int $nrEstrutLegal,
        string $attachment
    ) {
        /* Condicao temporaria */
        if ($this->environment->getNrorg() == 3260 || $this->environment->getNrorg() == 1410) {
            return;
        }
        
        $notSentEmails = [];

        foreach ($operadoresDP as $operDP) {
            $email = $operDP['DSEMAILOPER'];
            $name = $operDP['NMOPERADOR'];

            $emailEnviado = $this->sendEmail(
                $email,
                $subject,
                $name,
                $content,
                $nrEstrutLegal,
                EmailService::REMETENTE_DEFAULT,
                $attachment,
                false,
                EmailService::TEMPLATE_DEFAULT
            );

            if (!$emailEnviado) {
                array_push($notSentEmails, $name);
            }
        }

        if (!empty($notSentEmails)) {
            $pessoas = implode(', ', $notSentEmails);
            $message = "Não foi possível enviar e-mail para as seguintes pessoas: $pessoas!";
            throw new \Exception($message);
        }
    }

    public function sendEmailNovoOperador(
        string $nmPessoa,
        string $email,
        string $senha
    ): void {
        $content = EmailMessages::NOVO_OPERADOR_CONTENT;

        $params = [
            '%codigo%' => $email,
            '%email%' => $email,
            '%senha%' => $senha,
        ];

        $content = strtr($content, $params);
        $subject = 'Novos dados de acesso';
        $nrEstrutLegal = 151000063; // TEKNISA SERVICE LTDA.

        $this->sendEmail(
            $email,
            $subject,
            $nmPessoa,
            $content,
            $nrEstrutLegal,
            self::REMETENTE_DEFAULT
        );
    }
}