<?php

namespace Ems\Services;

use Util\Environment;
use Ems\Repository\MovimentacaoRepository;
use Util\NovoCodigo;
use Teknisa\Model\Gestaopessoas\Entities\GpeTipotransfer;
use Teknisa\Model\Geral\Entities\Tipoestrutura;
use Ems\Util\Utils;
use Util\{ Repositories };

class MovimentacaoService {
    /** @var Environment */
    protected $environment;

    /** @var MovimentacaoRepository */
    protected $movimentacaoRepository;

    public function __construct(
        Environment             $environment,
        MovimentacaoRepository  $movimentacaoRepository
    ) {
        $this->environment              = $environment;
        $this->movimentacaoRepository   = $movimentacaoRepository;
    }

    public function setMovimentacao(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRMOVIMENTACAO'] =
            $row['NRMOVIMENTACAO'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_MOVIMENTACAO', 1, 12, true, false);

        $params['NRORG']            = $row['NRORG'] ?? $nrOrg;
        $params['NRVINCULOM']       = (int) $row['NRVINCULOM'];
        $params['NRTIPOTRANSFER']   = (int) $row['NRTIPOTRANSFER'];
        $params['NRTPMOVTRANSFM']   = $row['NRTPMOVTRANSFM'] ?? null;
        $params['NRMOTIVOTRANSF']   = $row['NRMOTIVOTRANSF'] ?? null;
        $params['NRESTRUTURAM']     = (int) $row['NRESTRUTURAM'];
        $params['DTINIMOVIMENT']    = (string) $row['DTINIMOVIMENT'];
        $params['DTFIMMOVIMENT']    = $row['DTFIMMOVIMENT'] ?? null;
        $params['DSOBSERVACAO']     = $row['DSOBSERVACAO'] ?? null;
        $params['DTINCLUSAO']       = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']         = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']    = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']   = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']      = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']     = $row['CDOPERULTATU'] ?? null;
        $params['NRTIPOESTRUTURA']  = $row['NRTIPOESTRUTURA'] ?? null;
        $params['DTMOVIMENTRETRO']  = $row['DTMOVIMENTRETRO'] ?? null;
        $params['IDATUALIZAACERTO'] = $row['IDATUALIZAACERTO'] ?? 0;
        $params['NRMOTIVOAFASTA']   = $row['NRMOTIVOAFASTA'] ?? null;

        $this->movimentacaoRepository->setMovimentacao($params);

        return $params['NRMOVIMENTACAO'];
    }

    public function verificaTipoTransferencia($nrtipotransfer, $nrtipoestrutura){
        try {
            $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => Utils::getParamters()->getNrorg()));
            if ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL) {
                if ($nrtipoestrutura != $paramOrg->getNrtpestrutlegal() && $nrtipoestrutura != Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao legal incompatível com o tipo de estrutura', 30);
                }
            } elseif ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN) {
                if (($nrtipoestrutura == $paramOrg->getNrtpestrutlegal() && $paramOrg->getNrtpestrutlegal() != $paramOrg->getNrtpestrutgeren()) || $nrtipoestrutura == Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao gerencial incompatível com o tipo de estrutura', 30);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}