<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Ems\Repository\MarcacaoRepository;
use Ems\Repository\OcorrenciaRepository;
use Ems\Repository\PeriodoRepository;
use Ems\Services\MarcacaoService;
use Ems\Services\VinculoService;
use Ems\Services\EmailService;
use Ems\Services\ParceiroNegocioService;
use Ems\Services\ApuracaoService;
use Ems\Services\OperadorService;
use Ems\Services\ParamOrgService;
use Ems\Services\EstruturaService;
use Ems\Util\{ Utilities, Time, Utils, ExpectedException, ArrayUtil};
use Ems\Util\Utils as EmsUtil;
use Util\Environment;
use Util\DateUtil;
use Util\SendEmail\SendEmail;
use Util\Repositories;
use Util\EmailMessages;
use Util\Messages;
use Util\InstanceProvider;
use Util\Util;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Doctrine\DBAL\Connection;
use Folha\Entities\PtoProgocorrencia;

class OcorrenciaService {

    /** @var EntityManager */
    protected $entityManager;

    /** @var EmailService */
    protected $emailService;

    /** @var Utilities */
    protected $utilities;

    /** @var OcorrenciaRepository */
    protected $ocorrenciaRepository;

    /** @var OperadorService */
    protected $operadorService;

    /** @var ApuracaoService */
    protected $apuracaoService;

    /** @var VinculoService */
    protected $vinculoService;

    /** @var PeriodoRepository */
    protected $periodoRepository;

    /** @var Connection */
    protected $connection;

    /** @var Environment */
    protected $environment;

    /** @var MarcacaoRepository */
    protected $marcacaoRepository;

    /** @var array */
    private $ultimaOcorrencia;

     /** @var array */
     private $ocorrenciaRow;

    public function __construct(
        EntityManager              $entityManager,
        Environment                $environment,
        ParceiroNegocioService     $parceiroNegocioService,
        EmailService               $emailService,
        ParamOrgService            $paramOrgService,
        SituacaoFuncionalService   $situacaoFuncionalService,
        MarcacaoRepository         $marcacaoRepository,
        OcorrenciaRepository       $ocorrenciaRepository,
        VinculoService             $vinculoService,
        ApuracaoService            $apuracaoService,
        Utilities                  $utilities,
        OperadorService            $operadorService,
        PeriodoRepository          $periodoRepository,
        EstruturaService           $estruturaService
    ) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->emailService             = $emailService;
        $this->paramOrgService          = $paramOrgService;
        $this->situacaoFuncionalService = $situacaoFuncionalService;
        $this->marcacaoRepository       = $marcacaoRepository;
        $this->ocorrenciaRepository     = $ocorrenciaRepository;
        $this->vinculoService           = $vinculoService;
        $this->apuracaoService          = $apuracaoService;
        $this->utilities                = $utilities;
        $this->operadorService          = $operadorService;
        $this->periodoRepository        = $periodoRepository;
        $this->connection               = $entityManager->getConnection();
        $this->ultimaOcorrencia         = null;
        $this->estruturaService         = $estruturaService;

        // $this->marcacaoService          = InstanceProvider::getMarcacaoServiceInstance();
    }

    public const ocorrenciaDefaults = array(
        'ALMOCO' => array(
            'nrtipoprogocorrencia' => '20'
        )
    );

    /** @var string */
    const NRTIPOPROGOCORRENCIA_ALMOCO               = '20';

    const NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG  = '26';

    const HRPROJETARINTERVALONPROG_INT_NAO_PROG     = '0015';
    const HRPROJETARINTERVALONPROG_ALMOCO           = '0100';

    public const messages = array(
        // caso a mensagem seja alterada, modifique a função "insertAlmocoGestor" também
        'ERRO_OCORRENCIA_ALMOCO_INSERIDA' => 'Ocorrência de almoço já cadastratada para o dia @.',
    );

    // ID de status da Programação de ocorrência
    const IDPROGOCORCONFIRMADA_PENDENTE   = 'P';
    const IDPROGOCORCONFIRMADA_CONFIRMADA = 'S';
    const IDPROGOCORCONFIRMADA_REJEITADA  = 'N';

    const IDPROGOCORCONFIRMADA_PENDENTE_DESC   = 'Pendente';
    const IDPROGOCORCONFIRMADA_CONFIRMADA_DESC = 'Confirmada';
    const IDPROGOCORCONFIRMADA_REJEITADA_DESC  = 'Rejeitada';

    // Retorno de APIs
    const DEFAULT_ALREADYINCLUDED_RETURN   = 'ALREADY_INCLUDED';
    const DEFAULT_NOTSAVED_RETURN          = 'NOT_SAVED';
    const DEFAULT_INCLUDED_RETURN          = 'INSERTED';

    // Tipos ocorrencia
    const NRTIPOPROGOCORRENCIA_SOLICITACAOCOMPENSARFALTA  = '10';
    const NRTIPOPROGOCORRENCIA_SOLICITACAOABONARFALTA     = '5';
    const NRTIPOPROGOCORRENCIA_SOLICITACAOHORASEXTRAS     = '1';
    const NRTIPOPROGOCORRENCIA_TROCAHORARIOS              = '29';
    const NRTIPOPROGOCORRENCIA_AUTORIZARMARCACAO          = '30';

    // Motivos ocorrencia
    const           NRMOTIVOOCORFREQ_ATEST_COMP = 23;

    // ID's de horario
    const           IDTIPOHORARIO_FOLGA      = 'F';
    const           IDTIPOHORARIO_NORMAL     = 'N';
    const           IDTIPOHORARIO_COMPENSADO = 'C';
    const           IDTIPOHORARIO_FOLGA_COMP = [self::IDTIPOHORARIO_FOLGA, self::IDTIPOHORARIO_COMPENSADO];

    // ID's de tipos de dia de ocorrência
    const           IDTIPODIACALENDARIO_FERIADO = 'FERIADO';

    // Textos padrão
    protected const TEXTO_PADRAO_SOLICITACAO_HORAS_EXTRAS = 'Tempo de hora extra solicitado: @HORAS@ - ';

    // Flag que controla o envio dos emails quando houver alteração do status da ocorrência
    protected const ENVIA_EMAIL_ALTERACAO_STATUS_OCORRENCIA = true;

    // Flag que controla o comportamento padrão ao solicitar compensações de horas.
    const           COMPENSACAO_ATUALIZA_APURACOES_POSTERIORES_DEFAULT = true;

    // ID's de dia de semana
    protected const IDDIASSEMANA_DEFAULT = '1111111';

    const APROVAHE_VALIDA_EXISTENCIA_CALCULO = true;
    const APROVAHE_NAO_VALIDA_EXISTENCIA_CALCULO = false;

    const APROVAHE_EXIGE_APROVACAO = true;
    const APROVAHE_NAO_EXIGE_APROVACAO = false;

    /**
     * Função responsável pela criação de uma ocorrência relacionada a ausência de almoço durante a jornada
     */
     public function setOcorrenciaAlmoco(
        string  $nrvinculom,
        string  $nrorg,
        string  $nrmarcacaoponto,
        ?string $cdoperador,
        string  $dia = null
    ): int {
        try {
            $dataAtual = new \DateTime();
            if (empty($dia)) {
                $dia = $dataAtual->format(DateUtil::FORMATO_DATA);
            }

            $dtinicioprogocor       = $dia;
            $dadosHorarioVinculo    = $this->marcacaoRepository->getDadosHorarioVinculo($nrvinculom, $nrorg, $dia);
            $nrtipoprogocorrencia   = $this->getTipoOcorrenciaAlmocoPorJornada($nrvinculom, $dia, $dadosHorarioVinculo['IDTIPOHORARIO']);
            $idprogocorconfirmada   = $this->ocorrenciaRepository->getTipoProgocorrencia($nrorg, $nrtipoprogocorrencia);


            if (empty($dadosHorarioVinculo)) {
                throw new \Exception(Messages::OCOR_DADOS_HOR_VINC_NOT_FOUND);
            }

            $dtfimprogocor              = $dia;
            $iddiassemana               = EmsUtil::getIdDiaSemana($dtinicioprogocor);
            $idtipohorario              = $dadosHorarioVinculo['IDTIPOHORARIO'];
            $nrhordiam                  = $dadosHorarioVinculo['NRHORDIAM'];
            $nrintervalohorario         = $dadosHorarioVinculo['NRINTERVALOHORARIO'];
            $idtipocalendario           = '';
            $nrmotivoocorfreq           = '';
            $qtmaximaprogocor           = '';
            $dsobservacao               = '';
            $dsobservacaorescusa        = '';
            $hrprojetarintervalonprog   = $this->getHrptojetainteravaloByTipoOcorrencia($nrvinculom, $dia, $nrtipoprogocorrencia);
            $dtinclusao                 = $dataAtual->format(DateUtil::FORMATO_DATA);

            $insertResult = $this->ocorrenciaRepository->geraOcorrenciaFrequencia(
                $nrvinculom,
                $dtinicioprogocor,
                $dtfimprogocor,
                $iddiassemana,
                $idtipocalendario,
                $idtipohorario,
                $nrhordiam,
                $nrintervalohorario,
                $nrmotivoocorfreq,
                $qtmaximaprogocor,
                $dsobservacao,
                $nrtipoprogocorrencia,
                $nrmarcacaoponto,
                $idprogocorconfirmada,
                $dsobservacaorescusa,
                $hrprojetarintervalonprog,
                $dtinclusao,
                $cdoperador,
                $nrorg
            );


            if ($insertResult) {

                // Verificação para alterar marcações, se necessário.
                InstanceProvider::getMarcacaoServiceInstance()->updateMarcacoesIntervaloAlmoco($nrvinculom, $dia);
            }

            return $insertResult;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function setOcorrenciaAlmocoByNrmarcacao(
        int     $nrorg,
        int     $nrmarcacaoponto,
        string  $cdoperador
    ) {
        // Jogar essa função no marcacaoservice
        $marcacao = $this->marcacaoRepository->getMarcacaoByNrmarcacaoponto(
            $nrorg,
            $nrmarcacaoponto
        );

        $dia = DateUtil::removeTimeFromDateString($marcacao['DTHRMARCACAOPONTO']);

        $nrvinculom = $marcacao['NRVINCULOM'];

        return $this->setOcorrenciaAlmoco(
            $nrvinculom,
            $nrorg,
            $nrmarcacaoponto,
            $cdoperador,
            $dia
        );
    }

    // varre marcações e coloca na segunda marcação, se não for a última (primeira ausência), o intervalo de almoço
    public function insertDefaultAlmocoByDataMarcacao(
        int     $nrorg,
        int     $nrvinculom,
        string  $date,
        string  $cdoperador
    ) {
        $primeiroIntervaloMarcacao = $this->marcacaoRepository->getPrimeiraAusenciaMarcacaoDia(
            $nrorg,
            $nrvinculom,
            $date
        );

        $marcacaoIntervalo = $primeiroIntervaloMarcacao['NRMARCACAOPONTO'];

        return $this->setOcorrenciaAlmocoByNrmarcacao(
            $nrorg,
            $marcacaoIntervalo,
            $cdoperador
        );
    }

    public function getOcorrenciasPeriodo(string $nrorg, string $nrvinculom, string $nrperiodoapuracao, string $idprogocorconfirmada = null) {
        try {

            return $this->ocorrenciaRepository->getOcorrenciasPeriodo(
                $nrorg,
                [$nrvinculom],
                $nrperiodoapuracao,
                $idprogocorconfirmada
            );
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function getOcorrenciasDia(
        string  $nrorg,
        string  $nrvinculom,
                $dtinicioprogocor,
        string  $nrtipoprogocorrencia = null,
        string  $cdoperador           = null
    ) {

        try {

            $perfacesst = null;

            if(!empty($cdoperador)) {
                $perfacesst = $this->ocorrenciaRepository->verificaPerfilAcessoSESMT($nrorg, $cdoperador);
            }
            
            return $this->ocorrenciaRepository->getOcorrenciasDia(
                $nrorg,
                $nrvinculom,
                $dtinicioprogocor,
                $nrtipoprogocorrencia,
                $perfacesst
            );
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function getMotivoOcorFreq(
        $nrOrg = null,
        $cdOperador = null
    ) {
        if ($nrOrg === null) {
            $nrOrg = (int) $this->environment->getNrOrg();
        }

        $motivos = [];
        $operadorIsDp = !empty($cdOperador) && $this->operadorService->operadorIsDP($cdOperador);
        if (Utils::getParamters()->isRequestFromPortalGestor() && $operadorIsDp) {
            $motivos = $this->ocorrenciaRepository->getMotivoOcorFreqAdmin(
                $nrOrg
            );
        }else {
            $motivos = $this->ocorrenciaRepository->getMotivoOcorFreq(
                $nrOrg
            );
        }

        return $motivos;
    }
    
    public function getMotivoOcorFreqLancamentoRetroativo(
        $nrOrg = null,
        $cdOperador = null
    ) {
        if ($nrOrg === null) {
            $nrOrg = (int) $this->environment->getNrOrg();
        }
        
        if ($nrOrg == Utils::ORG_TEKNISA) {
            return $this->ocorrenciaRepository->getMotivoOcorFreqLancamentoRetroativoEspecificasTeknisa(
                $nrOrg
            );
        }
        
        return $this->getMotivoOcorFreq($nrOrg, $cdOperador);

    }

    public function getTpOcorrencia($nrorg) {
        try {
			return $this->ocorrenciaRepository->getTpOcorrencia(
                $nrorg
            );
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function setOcorrenciaFreq($row) {
        try {
            $this->ocorrenciaRow = $row;
            $this->ocorrenciaRow['__is_new'] = $this->ocorrenciaRow['__is_new'] ?? true;
            $ocorrenciaSaved = false;
            $previousOcorrenciaData = [];

            // Operações padrão na row
            $this->ocorrenciaRow['DSPATH'] = $this->saveFile($this->ocorrenciaRow);
            $this->ocorrenciaRow['IDDIASSEMANA'] = self::IDDIASSEMANA_DEFAULT;
            $dtInicioOcorSeguinte = !empty($this->ultimaOcorrencia) 
                ? DateUtil::adicionaIntervalo(DateUtil::getDataDeString($this->ultimaOcorrencia['DTFIMPROGOCOR'], DateUtil::FORMATO_DATA, true), 1, DateUtil::DIA)
                : null;
            $dtInicioUltimaOcor = DateUtil::getDataDeString($this->ocorrenciaRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_DATA, true);
            if (!empty($this->ultimaOcorrencia) &&
                $this->ocorrenciaRow['__is_new'] &&
                $this->ultimaOcorrencia['NRVINCULOM'] == $this->ocorrenciaRow['NRVINCULOM'] &&
                $this->ultimaOcorrencia['NRTIPOPROGOCORRENCIA'] == $this->ocorrenciaRow['NRTIPOPROGOCORRENCIA'] &&
                DateUtil::comparaDatas($dtInicioOcorSeguinte, $dtInicioUltimaOcor, DateUtil::IGUAL)
            ){ 
                $this->ocorrenciaRow['DTINICIOPROGOCOR'] = $this->ultimaOcorrencia['DTINICIOPROGOCOR'];
                $this->ocorrenciaRow['NRPROGOCORRENCIA'] = $this->ultimaOcorrencia['NRPROGOCORRENCIA'];
                $this->ocorrenciaRow['IDPROGOCORCONFIRMADA'] = $this->ultimaOcorrencia['IDPROGOCORCONFIRMADA'];
                $this->ocorrenciaRow['__is_new'] = false;
                
            }

            // Salva as ocorrencias
            if($this->ocorrenciaRow['__is_new']) {
                $ocorrenciaSaved = $this->newOcorrenciaFreq();
                $this->ocorrenciaRow['NRPROGOCORRENCIA'] = $ocorrenciaSaved;
            } else {
                $previousOcorrenciaData = $this->ocorrenciaRepository->getOcorrencia($this->ocorrenciaRow['NRORG'], $this->ocorrenciaRow['NRPROGOCORRENCIA']);
                $ocorrenciaSaved = $this->ocorrenciaRepository->updateOcorrenciaFreq($this->ocorrenciaRow);
            }

            return $ocorrenciaSaved;
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function setOcorrenciasFreq(
                $row,
        array   $dates
    ) {
        try {

            $dates = DateUtil::sortDateArray($dates);

            foreach ($dates as $date) {

                $marcacao = $this->marcacaoRepository->getMarcacaoByDataMarcacao(
                    $row['NRORG'],
                    $row['NRVINCULOM'],
                    $row['HORAMARCACAO'],
                    $date
                );

                if (empty($marcacao)) {
                    throw new \Exception(Messages::MARCACAO_NOT_FOUND);
                }

                $row['NRMARCACAOPONTO']     = $marcacao['NRMARCACAOPONTO'];
                $row['DTINICIOPROGOCOR']    = $date;
                $row['DTFIMPROGOCOR']       = $date;

                $success = $this->setOcorrenciaFreq($row);

                if (empty($success)) {
                    throw new \Exception(Messages::OCOR_ERR_NOT_SAVED);
                }
            }

            return true;

		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function newOcorrenciaFreq($row = null) {
        $this->ocorrenciaRow = $row ?? $this->ocorrenciaRow;
        $nrOrg = (int) $this->environment->getNrOrg();

        if (!isset($this->ocorrenciaRow['IDPROGOCORCONFIRMADA']) && isset($this->ocorrenciaRow['NRTIPOPROGOCORRENCIA'])) {
            $nrTipoOcor = $this->ocorrenciaRow['NRTIPOPROGOCORRENCIA'];

            $idConfirmada = $this->ocorrenciaRepository->getTipoProgocorrencia($nrOrg, $nrTipoOcor);
            $this->ocorrenciaRow['IDPROGOCORCONFIRMADA'] = $idConfirmada;
        }

        $nrVinculo = $this->ocorrenciaRow['NRVINCULOM'];
        $date = $this->ocorrenciaRow['DTINICIOPROGOCOR'];

        $horarioBase = $this->vinculoService->getHorarioBase(
            $nrVinculo, $date, VinculoService::GETHORARIOBASE_NOT_THROW_EXCEPTION
        );

        $idTipoHorario = $horarioBase['IDTIPOHORARIO'] ?? 'N';
        $this->ocorrenciaRow['IDTIPOHORARIO'] = $this->ocorrenciaRow['IDTIPOHORARIO'] ?? $idTipoHorario;

        $success = $this->ocorrenciaRepository->newOcorrenciaFreq($this->ocorrenciaRow);

        if (isset($this->ocorrenciaRow['DSPATH'])) {
            try {
                $this->sendEmailOcorrencia($this->ocorrenciaRow);
            } catch (\Exception $e) {
                //
            }
        }

        return $success;
    }

    public function saveFile($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        if (isset($row['FILE']) && $row['FILE'] !== '') {
            $file = $row['FILE'];

            $filePath = $this->utilities->saveFileOnFileServer(
                $file, 'jpeg', $nrOrg, null, null, '', false
            );



            return $filePath;
        }

        return NULL;
    }

    private function sendEmailOcorrencia(
        array $row
    ) {
        if (!isset($row['NMFILE'])) {
            return;
        }

        $cdOperador = $row['CDOPERADOR'] ?? '1';
        $nrVinculo = (int) $row['NRVINCULOM'];
        $file = (string) $row['FILE'];
        $fileName = (string) $row['NMFILE'];

        $operador = $this->operadorService->get($cdOperador);
        $vinculo = $this->vinculoService->getDadosVinculo($nrVinculo);
        $operadoresDP = $this->operadorService->getOperadoresDp();

        $nmOperador = (string) $operador['NMOPERADOR'];
        $nmVinculo = (string) $vinculo['NMVINCULOM'];
        $nrEstrutLegal = $vinculo['NRESTRUTLEGAL'];

        if ($nrEstrutLegal === null) {
            $message = "O vínculo $nmVinculo não possui estrutura legal!";
            throw new ExpectedException($message);
        }

        $translationSubject = [
            '@NOME_VINCULO@' => $nmVinculo,
        ];

        $subjectTemplate = EmailMessages::ALTERSTATUSOCOR_SUBJECT;
        $subject = strtr($subjectTemplate, $translationSubject);

        $content = $this->getEmailContent(
            $nmOperador, $nmVinculo, $row
        );

        $attachment = Utilities::blobToFile(
            $file, $fileName
        );

        $this->emailService->enviarEmailsOcorrenciaDocumento(
            $operadoresDP, $subject, $content,
            $nrEstrutLegal, $attachment
        );
    }

    private function getEmailContent(
        string $nmOperador,
        string $nmVinculo,
        array $row
    ): string {
        $date = date('d/m/Y');
        $time = date('H:i');

        $dsConfirmada = Utils::getIdOcorrenciaDescription(
            $row['IDPROGOCORCONFIRMADA']
        );

        $dtIni = (string) $row['DTINICIOPROGOCOR'];
        $dtFim = (string) $row['DTFIMPROGOCOR'];
        $observacao = $row['DSOBSERVACAO'] ?? '';
        $nrMotivo = !empty($row['NRMOTIVOOCORFREQ']) ? ((int) $row['NRMOTIVOOCORFREQ']) : null;
        $dsMotivo = '';

        if (!empty($nrMotivo)) {
            $motivo = $this->getMotivo($nrMotivo);
            $dsMotivo = $motivo['DSMOTIVOOCORFREQ'];
        }

        $translations = [
            '@IDPROGOCORCONFIRMADA@'    => $dsConfirmada,
            '@NOME_GESTOR@'             => $nmOperador,
            '@DATA_ALTERACAOSTATUS@'    => $date,
            '@HORA_ALTERACAOSTATUS@'    => $time,
            '@DTINICIOPROGOCOR@'        => $dtIni,
            '@DTFIMPROGOCOR@'           => $dtFim,
            '@NOME_VINCULO@'            => $nmVinculo,
            '@DSMOTIVOOCORFREQ@'        => $dsMotivo,
            '@DSOBSERVACAO@'            => $observacao,
        ];

        $contentTemplate = EmailMessages::ALTERSTATUSOCOR_BODYMESSAGE;

        $content = strtr($contentTemplate, $translations);

        return $content;
    }

    public function removeOcorrenciaFreq($row) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOcorrencia = (int) $row['NRPROGOCORRENCIA'];
        $ocorrenciaDeleted = false;

        $ocorrencia = $this->ocorrenciaRepository->
            getOcorrencia($nrOrg, $nrOcorrencia);

        if ($ocorrencia) {
            if (isset($ocorrencia['DSPATH']) && $ocorrencia['DSPATH'] !== '') {
                $this->utilities->deleteFile($ocorrencia['DSPATH']);
            }
        }

        $ocorrenciaDeleted = $this->ocorrenciaRepository->removeOcorrenciaFreq($row);
        if ($ocorrenciaDeleted) {
            $this->situacaoFuncionalService->afterAlterOcorrenciaSetAlteSituFunc(
                $row['NRVINCULOM'],
                [],
                $row,
                Utils::DATA_OPERATION_DELETE
            );
        }

        return $ocorrenciaDeleted;
    }

    public function removeOcorrenciasFreq($row, $dates) {
        try {

            foreach ($dates as $date) {

                $ocorrencia = $this->getOcorrenciaByDateMarcacao(
                    $row['NRORG'],
                    $row['NRVINCULOM'],
                    $row['HORAMARCACAO'],
                    $date
                );

                if (empty($ocorrencia)) {
                    throw new \Exception(Messages::OCOR_ERR_NOT_FOUND);
                }

                $row['NRPROGOCORRENCIA'] = $ocorrencia['NRPROGOCORRENCIA'];

                $success = $this->removeOcorrenciaFreq($row);

                if (empty($success)) {
                    throw new \Exception(Messages::OCOR_ERR_NOT_DELETED);
                }
            }

            return true;

        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function existemOcorrenciasAlmoco(
        string  $nrvinculom,
        string  $nrorg,
        string  $dtinicioprogocor,
        string  $nrmarcacaoponto = null
    ): bool {

        $nrtipoprogocorrencia = self::NRTIPOPROGOCORRENCIA_ALMOCO;
        $ocorrenciasAlmocoDia = $this->ocorrenciaRepository->getOcorrenciasPorTipoOcorrencia($nrvinculom, $nrorg, $nrtipoprogocorrencia, $dtinicioprogocor, $nrmarcacaoponto);

        $existemOcorrenciasAlmocoDia = !empty($ocorrenciasAlmocoDia);

        return $existemOcorrenciasAlmocoDia;
    }

    public function insertAlmocoGestor(
        string  $cdoperador,
        int     $nrVinculo,
        string  $date,
        int     $nrMarcacao
    ): bool {
        $nrOrg = (int) $this->environment->getNrOrg();

        try {
            $this->setOcorrenciaAlmoco($nrVinculo, $nrOrg, $nrMarcacao, $cdoperador, $date);
        } catch(\Exception $e) {
            $alreadyHasAlmoco =
                strpos($e->getMessage(), 'Ocorrência de almoço já cadastratada') !== false;

            if ($alreadyHasAlmoco) {
                return false;
            }

            throw $e;
        }

        return true;
    }

    public function insertAlmocoMultiplesGestor(
        string  $cdoperador,
        int     $nrVinculo,
        array   $dates
    ): bool {
        $nrOrg = (int) $this->environment->getNrOrg();

        foreach ($dates as $date) {

            $success = $this->insertDefaultAlmocoByDataMarcacao(
                $nrOrg,
                $nrVinculo,
                $date,
                $cdoperador
            );

            if (empty($success)) {
                return false;
            }

        }

        return true;
    }


    public function getOcorrenciasPendentesPeriodoVinculosGestor(string $nrestruturam, string $nrperiodoapuracao, $cdoperador = null) {
        $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_PENDENTE;

        return $this->getOcorrenciasVinculosGestor($nrestruturam, $nrperiodoapuracao, $idprogocorconfirmada, $cdoperador);
    }

    public function getOcorrenciasVinculosGestor(int $nrestruturam, int $nrperiodoapuracao, string $idprogocorconfirmada = null, $cdoperador = null): Array {
        $nrorg = $this->environment->getNrorg();
        $perfacesst = null;

        // Vinculos gerenciados pelo gestor

        /*
        * $vinculoGestor = $this->vinculoService->getVinculosGestor($nrestruturam, $nrperiodoapuracao);
        * $nrvinculomVinculosGestor = array_map(function ($vinculo) { return (int) $vinculo['NRVINCULOM']; }, $vinculoGestor);
        *
        * \\ Modificado para pegar estruturas ao invés de vínculo para evitar a quebra do instr para organizações grandes
        */    

        $estruturas = $this->getNrEstruturas($nrestruturam);
        
        if(!empty($cdoperador)) {
            $perfacesst = $this->ocorrenciaRepository->verificaPerfilAcessoSESMT($nrorg, $cdoperador);
        }
        
        $ocorrenciasVinculosGestor = $this->ocorrenciaRepository->getOcorrenciasPeriodoPorEstrutura($nrorg, $estruturas, $nrperiodoapuracao, $idprogocorconfirmada, $perfacesst);
        $ocorrenciasVinculosGestor = $this->addHorarioFinalProp($ocorrenciasVinculosGestor);

        return $ocorrenciasVinculosGestor;

    }

    public function addHorarioFinalProp(
        array $ocorrencias
    ): array {
        /** @var MarcacaoService */
        $marcacaoService = InstanceProvider::getMarcacaoServiceInstance();
        $tiposOcorrenciaExcecoes = ['30'];

        foreach ($ocorrencias as &$ocor) {
            if (in_array($ocor['NRTIPOPROGOCORRENCIA'], $tiposOcorrenciaExcecoes)) {
                continue;
            }

            $nrMarcacao = $ocor['NRMARCACAOPONTO'];

            if ($nrMarcacao === null) {
                continue;
            }

            $nrVinculo = (int) $ocor['NRVINCULOM'];

            $marcacaoSeguinte = $marcacaoService->getMarcacaoSeguinte(
                $nrVinculo, $nrMarcacao
            );

            if ($marcacaoSeguinte === false) {
                continue;
            }

            $horaMarcacao = (string) $marcacaoSeguinte['HORA_MINUTO'];

            $ocor['HRFIM'] = $horaMarcacao;
        }

        return $ocorrencias;
    }

    /**
     * Rotina que altera o status de uma ocorrência PENDENTE. Pode propiciar o disparo de emails para os parceiros de negocio com perfil de acesso associado a organização por meio do parâmetro $disparaemail
     *
     * @param string $nrprogocorrencia
     * @param string $idprogocorconfirmada
     * @param bool   $disparaemail          Default: false,
     * @param string $cdoperador            Default: null
     */
    public function alteraStatusOcorrencia(
        string $nrprogocorrencia,
        string $idprogocorconfirmada,
        bool $disparaemail = false,
        string $cdoperador = null
    ) {
        $nrorg = $this->environment->getNrorg();

        $this->verifyPermission($cdoperador);

        $ocorrenciaAlterada = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrorg, 'nrprogocorrencia' => $nrprogocorrencia, 'idprogocorconfirmada' => self::IDPROGOCORCONFIRMADA_PENDENTE));

        $operador = $this->operadorService->get($cdoperador);

        if ($ocorrenciaAlterada) {
            $ocorrenciaAlterada->setIdprogocorconfirmada($idprogocorconfirmada);
            $ocorrenciaAlterada->setCdoperultatu($cdoperador);
            $ocorrenciaAlterada->setDtultatu(DateUtil::getDataAtual(false));
            $ocorrenciaAlterada->setNrorgultatu($nrorg);

            $observacao = $ocorrenciaAlterada->getDsobservacao();
            $novaObservacao = (empty($observacao) ? '' : $observacao . PHP_EOL) .
                'Processada por: ' . $operador['NMOPERADOR'] . ', data: ' .  DateUtil::getDataAtual(false);

            $ocorrenciaAlterada->setDsobservacao($novaObservacao);

            $this->entityManager->flush();

            $this->ocorrenciaAlterada = $ocorrenciaAlterada;

            if($nrorg == '20' && $ocorrenciaAlterada->getNrmotivoocorfreq() == '36') {
                return true;
            } else {
                if ($disparaemail && !empty($cdoperador)) {
                    // Dispara email com as alterações realizadas
                    $emailsEnviados = $this->enviaEmailAlteracaoStatusOcorrencia($ocorrenciaAlterada, $cdoperador);
    
                    // Se os emails nao foram enviados, $emailsEnviados contem mais detalhes
                    if ($emailsEnviados !== true) {
                        $this->emailService->agrupaMensagensEmail($emailsEnviados);
                    }
                }
                return true;
            }
        }

        // Se chegou a esse ponto sem ter retornado, a ocorrência nao foi encontrada
        throw new \Exception(Messages::OCOR_NOT_FOUND);
    }

    public function aprovarOcorrencia(string $nrprogocorrencia, string $cdoperador, $row) {
        $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_CONFIRMADA;

        $result = $this->alteraStatusOcorrencia(
            $nrprogocorrencia,
            $idprogocorconfirmada,
            self::ENVIA_EMAIL_ALTERACAO_STATUS_OCORRENCIA,
            $cdoperador
        );

        $row['NRVINCULOM'] = $this->ocorrenciaAlterada->getNrVinculom();
        $row['NRMOTIVOOCORFREQ'] = $this->ocorrenciaAlterada->getNrmotivoOcorfreq();
        $row['DTINICIOPROGOCOR'] = $this->ocorrenciaAlterada->getDtInicioProgocor();
        $row['DTFIMPROGOCOR'] = $this->ocorrenciaAlterada->getDtFimProgocor();
        
        // After save
        if ($this->ocorrenciaAlterada) {
            $this->ultimaOcorrencia = $this->ocorrenciaAlterada;

            // Cadastro automatico de alteracoes de situacao funcional
            $operation = Utils::DATA_OPERATION_INSERT;
            $this->situacaoFuncionalService->afterAlterOcorrenciaSetAlteSituFunc(
                $row['NRVINCULOM'],
                $row,
                $row,
                $operation
            );          
        }

        return $result;
    }

    public function rejeitarOcorrencia(string $nrprogocorrencia, string $cdoperador) {
        $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_REJEITADA;

        $ocorrenciaAlterada = $this->alteraStatusOcorrencia(
            $nrprogocorrencia,
            $idprogocorconfirmada,
            self::ENVIA_EMAIL_ALTERACAO_STATUS_OCORRENCIA,
            $cdoperador
        );

        return $ocorrenciaAlterada;
    }

    public function enviaEmailAlteracaoStatusOcorrencia(
        PtoProgocorrencia $ocorrenciaAlterada,
        string $cdoperador
    ) {
        $nrorg = $this->environment->getNrorg();
        $momentoAlteracao = new \DateTime();
        $motivoocorr = $this->entityManager->getRepository(Repositories::PTO_MOTIVOOCORFREQ)->findOneBy(array('nrorg' => $nrorg, 'nrmotivoocorfreq' => $ocorrenciaAlterada->getNrmotivoocorfreq()));

        $idprogocorconfirmada = $ocorrenciaAlterada->getIdprogocorconfirmada();
        $dtinicioprogocor = $ocorrenciaAlterada->getDtinicioprogocor()->format(DateUtil::FORMATO_DATA);
        $dtfimprogocor = $ocorrenciaAlterada->getDtfimprogocor()->format(DateUtil::FORMATO_DATA);
        $dsmotivoocorfreq = !empty($motivoocorr) ? $motivoocorr->getDsmotivoocorfreq() : '';
        $dsobservacao = $ocorrenciaAlterada->getDsobservacao();
        $dataalteracaostatus = $momentoAlteracao->format(DateUtil::FORMATO_DATA);
        $horaalteracaostatus = $momentoAlteracao->format(DateUtil::FORMATO_HORA);

        $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrorg, 'nrvinculom' => $ocorrenciaAlterada->getNrvinculom()));
        $nomevinculo = $vinculom->getNmvinculom();

        switch ($idprogocorconfirmada) {
            case self::IDPROGOCORCONFIRMADA_CONFIRMADA:
                $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_CONFIRMADA_DESC;
                break;

            case self::IDPROGOCORCONFIRMADA_REJEITADA:
                $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_REJEITADA_DESC;
                break;

            default:
                $idprogocorconfirmada = self::IDPROGOCORCONFIRMADA_PENDENTE_DESC;
        }

        return $this->emailService->sendEmailNotificacaoOcorrencia(
            $cdoperador,
            array(
                'IDPROGOCORCONFIRMADA'  => $idprogocorconfirmada,
                'DTINICIOPROGOCOR'      => $dtinicioprogocor,
                'DTFIMPROGOCOR'         => $dtfimprogocor,
                'DSMOTIVOOCORFREQ'      => $dsmotivoocorfreq,
                'DSOBSERVACAO'          => $dsobservacao,
                'DATA_ALTERACAOSTATUS'  => $dataalteracaostatus,
                'HORA_ALTERACAOSTATUS'  => $horaalteracaostatus,
                'NOME_VINCULO'          => $nomevinculo
            ),
            EmailMessages::ALTERSTATUSOCOR_BODYMESSAGE,
            EmailMessages::ALTERSTATUSOCOR_SUBJECT
        );
    }

    /**
     *  Controller genérico como getter de ocorrências
     *  (Não terminado)
     */
    public function getOcorrencias(
        string $nrvinculom,
        string $nrperiodoapuracao = null,
        string $idprogocorconfirmada = null,
        string $dtinicioprogocor = null,
        string $dtfimprogocor = null
    ) {
        try {
            $nrorg = $this->environment->getNrorg();

            return $this->ocorrenciaRepository->getOcorrenciasPeriodo(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao,
                $idprogocorconfirmada,
                $dtinicioprogocor,
                $dtfimprogocor
            );
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function getAlmoco(
        int $nrVinculo,
        string $date
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'     => $nrOrg,
            'P_NRVINCULO' => $nrVinculo,
            'P_DATE'      => $date
        ];

        $almoco = $this->ocorrenciaRepository->getAlmoco($params);

        return $almoco;
    }

    public function removeAlmocoGestor(
        int $nrVinculo,
        string $date
    ): bool {
        $nrOrg = (int) $this->environment->getNrOrg();

        $almoco = $this->getAlmoco($nrVinculo, $date);

        if ($almoco === false) {
            return false;
        }

        $nrOcorrencia = $almoco['NRPROGOCORRENCIA'];

        $params = [
            'NRPROGOCORRENCIA' => $nrOcorrencia,
            'NRORG'      => $nrOrg,
            'NRVINCULOM' => $nrVinculo,
        ];

        $this->removeOcorrenciaFreq($params);

        return true;
    }

    /**
     * Rotina que remove a ocorrência de ALMOÇO a partir das DATAS e do VÍNCULO passados
     *
     *  @param          $nrVinculo,
     *  @param  Array   $dates
     *
     */
    public function removeAlmocos(
                $nrVinculo,
        array   $dates
    ) {
        try {
            $nrOrg = $this->environment->getNrOrg();

            foreach ($dates as $date) {

                $this->verificarPeriodo($nrVinculo, $date);

                $success = $this->removeAlmocoGestor(
                    $nrVinculo,
                    $date
                );

                if (empty($success)) {
                    throw new \Exception(Messages::OCOR_ALMOCO_NOT_REMOVED);
                }

            }

            return true;


        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciaFile(
        int $nrOcorrencia
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $ocorrencia = $this->ocorrenciaRepository->getOcorrencia(
            $nrOrg, $nrOcorrencia
        );

        if (!$ocorrencia) {
            return '';
        }

        $path = isset($ocorrencia['DSPATH']) ?
            $ocorrencia['DSPATH'] :
            '';

        if ($path === '') {
            return '';
        }

        $file = $this->utilities->downloadFile($path);

        return $file;
    }

    /**
     * Rotina que insere ocorrências a partir da data e da hora de uma MARCAÇÃO DE PONTO passada.
     *
     *  @param          $nrvinculom,
     *  @param          $cdoperador,
     *  @param          $horaMarcacao,
     *  @param  Array   $dates
     *
     */
    public function salvaOcorrenciasAlmocoByDataMarcacao(
                $nrvinculom,
                $cdoperador,
                $horaMarcacao,
        Array   $dates
    ) {
        try {
            $nrorg = $this->environment->getNrorg();
            $messages = array();

            foreach ($dates as $date) {
                $diaMarcacao = $date;

                try {
                    $success = $this->salvaOcorrenciaAlmocoByDataMarcacao(
                        $nrorg,
                        $nrvinculom,
                        $cdoperador,
                        $horaMarcacao,
                        $diaMarcacao
                    );

                    if (empty($success)) {
                        throw new \Exception("Ocorrência de almoço não registrada a partir do dia $date");
                    }
                } catch (\Exception $e) {
                    array_push($messages, $e->getMessage());
                }

            }

            if (!empty($messages)) {
                throw new \Exception(implode('<br>', $messages));
            }

            return true;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvaOcorrenciaAlmocoByDataMarcacao(
        $nrorg,
        $nrvinculom,
        $cdoperador,
        $horaMarcacao,
        $diaMarcacao
    ) {
        $marcacao = $this->marcacaoRepository->getMarcacaoByDataMarcacao(
            $nrorg,
            $nrvinculom,
            $horaMarcacao,
            $diaMarcacao
        );

        if (empty($marcacao)) {
            throw new \Exception(Messages::MARCACAO_NOT_FOUND);
        }

        $this->verificarPeriodo($nrvinculom, $diaMarcacao);

        $nrMarcacao = $marcacao['NRMARCACAOPONTO'];

        $success = $this->setOcorrenciaAlmoco(
            $nrvinculom,
            $nrorg,
            $nrMarcacao,
            $cdoperador,
            $diaMarcacao
        );

        return $success;
    }

    /**
     * Rotina que busca resgitro de ocorrência a partir do NRMARCACAOPONTO
     *
     *  @param  $nrMarcacao
     *
     */
    public function getOcorrenciaByNrmarcacao(
        $nrMarcacao
    ) {
        $nrOrg = $this->environment->getNrorg();

        return $this->ocorrenciaRepository->getOcorrenciaByNrmarcacao(
            $nrOrg,
            $nrMarcacao
        );
    }

    /**
     * Rotina que busca resgitro de ocorrência a partir da DATA DA MARCAÇÃO DE PONTO
     *
     *  @param          $nrorg,
     *  @param          $nrvinculom,
     *  @param string   $horamarcacao,
     *  @param string   $date
     */
    public function getOcorrenciaByDateMarcacao(
                $nrorg,
                $nrvinculom,
        string  $horamarcacao,
        string  $date
    ) {
        $marcacoes = $this->marcacaoRepository->getMarcacaoByDataMarcacao(
            $nrorg,
            $nrvinculom,
            $horamarcacao,
            $date,
            true
        );

        if (empty($marcacoes)) {
            throw new \Exception(Messages::MARCACAO_NOT_FOUND);
        }

        $ocorrencia = false;

        foreach ($marcacoes as $marcacao) {
            $nrmarcacaoponto = $marcacao['NRMARCACAOPONTO'];

            $ocorrencia = $this->getOcorrenciaByNrmarcacao(
                $nrmarcacaoponto
            );

            if ($ocorrencia) {
                break;
            }
        }

        return $ocorrencia;
    }

    public function removeOcorrenciaByNrmarcacao(
        $nrMarcacao
    ) {
        $nrorg = $this->environment->getNrorg();

        return $this->ocorrenciaRepository->removeOcorrenciaByNrmarcacao(
            $nrorg,
            $nrMarcacao
        );

    }

    /**
     * Rotina que gera a ocorrência de SOLICITAÇÃO DE COMPENSAÇÃO DE FALTA a partir das DATAS e do VÍNCULO passados
     *
     *  @param          $nrVinculo,
     *  @param  Array   $dates
     *
     */
    public function solicitaCompensacaoFaltas(
        int   $nrVinculo,
        array $dates
    ): array {
        $status = [
            'success' => true,
            'message' => null
        ];

        $atualizaApuracoesPosterioresCadaDia = false;

        if (empty($dates)) {
            $status['message'] = 'Nenhum dia foi encontrado para compensação';
            $status['success'] = false;
            return $status;
        }

        $message = '';
        
        $dates = DateUtil::sortDateArray($dates);

        foreach ($dates as $date) {
            $statusDia = $this->solicitaCompensacaoDia(
                $nrVinculo,
                $date,
                true,
                $atualizaApuracoesPosterioresCadaDia
            );

            if ($message !== '') {
                $message .= ' \n ';
            }

            if ($statusDia['success']) {
                $message .= "Dia $date compensado com sucesso!";
            } else {
                $status['success'] = false;
                $message .= $statusDia['message'];
            }
        }

        if (!$status['success']) {
            $status['message'] = $message;
        } else {
            // Atualiza apurações de dias posteriores ao primeiro dia compensado. Os demais, como têm apuracação, também serão reapurados.
            $primeiroDiaCompensacao = DateUtil::sortDateArray($dates)[0];
            $this->apuracaoService->atualizaApuracaoDiasPeriodo($nrVinculo, $primeiroDiaCompensacao);
        }

        return $status;
    }

    /**
     * Rotina que gera a ocorrência de SOLICITAÇÃO DE ABONO DE FALTA a partir das DATAS e do VÍNCULO passados
     *
     *  @param          $nrVinculo,
     *  @param  Array   $dates
     *
     */
    public function solicitaAbonoFaltas(
                $nrVinculo,
        array   $dates,
        string  $dsobservacao,
        string  $file,
        string  $nmFile,
        string  $nrMotivo,
        string  $cdTabecdi,
        string  $cdDiagnost,
        string  $nrPessoa
    ) {
        try {

            $diasSolicitacoesNaoInseridas = array();
            $diasSolicitacoesJaInseridas = array();
            $insercaoCompensacaoLevantaOcorrencia = false;

            $nrOrg = $this->environment->getNrorg();
            
            $dates = DateUtil::sortDateArray($dates);

            foreach ($dates as $date) {

                $solcitacaoInseridaComSucesso = $this->solicitaAbonoFalta(
                    $nrOrg,
                    $nrVinculo,
                    $date,
                    $dsobservacao,
                    $file,
                    $nmFile,
                    $insercaoCompensacaoLevantaOcorrencia,
                    $nrMotivo,
                    $cdTabecdi,
                    $cdDiagnost,
                    $nrPessoa
                );

                $paramsRespostaInsercao = $solcitacaoInseridaComSucesso[$date];

                $solicitacaoNaoInserida = $paramsRespostaInsercao === self::DEFAULT_NOTSAVED_RETURN;

                if ($solicitacaoNaoInserida) {
                    throw new \Exception(Messages::OCOR_ABONO_FALTA_NOT_SAVED);
                }

                $solicitacaoJaInserida = $paramsRespostaInsercao === self::DEFAULT_ALREADYINCLUDED_RETURN;

                if ($solicitacaoJaInserida) {
                    array_push($diasSolicitacoesJaInseridas, $date);

                    // break for loop
                    continue;
                }


                $solicitacaoNaoSalva = $paramsRespostaInsercao !== self::DEFAULT_INCLUDED_RETURN;

                if ($solicitacaoNaoSalva) {
                    throw new \Exception(Messages::OCOR_ABONO_FALTA_NOT_SAVED);
                }

            }

            $solicitacoesJaforamInseridas = !empty($diasSolicitacoesJaInseridas);

            if ($solicitacoesJaforamInseridas) {

                $todasSolicitacoesForamIncluidas = $dates === $diasSolicitacoesJaInseridas;
                if ($todasSolicitacoesForamIncluidas) {
                    throw new \Exception(Messages::OCOR_ABONO_FALTA_ALL_INCLUDED);
                }

                $messageParams = array('@DATES@' => implode(', ', $diasSolicitacoesJaInseridas));
                throw new \Exception(strtr(Messages::OCOR_ABONO_FALTA_ALREADY_INCLUDED_SPECIFIED_DATES, $messageParams));
            }

            return true;


        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function solicitaCompensacaoDia(
        int     $nrVinculo,
        string  $date,
        bool    $diaInteiro                     = true,
        bool    $atualizaApuracoesPosteriores   = self::COMPENSACAO_ATUALIZA_APURACOES_POSTERIORES_DEFAULT
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $status = $this->verifyCompensarDia(
            $nrVinculo,
            $date,
            $diaInteiro
        );

        if (!$status['success']) {
            return $status;
        }

        $iddiassemana = EmsUtil::getIdDiaSemana($date);

        $newOcorrencia = array(
            'NRTIPOPROGOCORRENCIA'  => self::NRTIPOPROGOCORRENCIA_SOLICITACAOCOMPENSARFALTA,
            'NRVINCULOM'            => $nrVinculo,
            'DTINICIOPROGOCOR'      => $date,
            'DTFIMPROGOCOR'         => $date,
            'NRORG'                 => $nrOrg,
            'IDDIASSEMANA'          => $iddiassemana,
        );

        $insertStatusOk = $this->setOcorrenciaFreq(
            $newOcorrencia
        );

        if ($insertStatusOk && $atualizaApuracoesPosteriores) { // Se a ocorrência foi inserida com sucesso
            // Atualiza apuração dias posteriores
            $this->apuracaoService->atualizaApuracaoDiasPeriodo($nrVinculo, $date);
        }

        return $status;
    }

    public function solicitaAbonoFalta(
                $nrOrg,
                $nrVinculo,
        string  $date,
        string  $dsobservacao,
        string  $file,
        string  $nmFile,
        bool    $throwsExceptionWhenAlreadyIncluded = true,
        string  $nrMotivo,
        string  $cdTabecdi,
        string  $cdDiagnost,
        string  $nrPessoa
    ) {
        try {

            $defaultAlreadyIncludedReturn   = self::DEFAULT_ALREADYINCLUDED_RETURN;
            $defaultNotSavedReturn          = self::DEFAULT_NOTSAVED_RETURN;
            $defaultIncludedReturn          = self::DEFAULT_INCLUDED_RETURN;

            $jaFoiSolicitadaCompensacao = $this->verificaSolicitacaoAbonoFaltaJaIncluida(
                $nrOrg,
                $nrVinculo,
                $date
            );

            if ($jaFoiSolicitadaCompensacao) {

                if ($throwsExceptionWhenAlreadyIncluded) {
                    throw new \Exception(Messages::OCOR_ABONO_FALTA_ALREADY_INCLUDED);
                }

                return [$date => $defaultAlreadyIncludedReturn];
            }

            $iddiassemana = EmsUtil::getIdDiaSemana($date);

            $newOcorrencia = [
                'NRTIPOPROGOCORRENCIA'  => self::NRTIPOPROGOCORRENCIA_SOLICITACAOABONARFALTA,
                'NRVINCULOM'            => $nrVinculo,
                'DTINICIOPROGOCOR'      => $date,
                'DTFIMPROGOCOR'         => $date,
                'NRORG'                 => $nrOrg,
                'IDDIASSEMANA'          => $iddiassemana,
                'DSOBSERVACAO'          => $dsobservacao,
                'FILE'                  => $file,
                'NMFILE'                => $nmFile,
                '__is_new'              => true,
                'NRMOTIVOOCORFREQ'      => $nrMotivo,
                'CDTABECDI'             => $cdTabecdi,
                'CDDIAGNOST'            => $cdDiagnost,
                'NRPESSOA'              => $nrPessoa,
            ];

            try {

                $this->setOcorrenciaFreq(
                    $newOcorrencia
                );

            } catch (\Exception $e) {
                return [$date => $defaultNotSavedReturn];
            }


            return [$date => $defaultIncludedReturn];


        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function verificaSolicitacaoCompensacaoFaltaJaIncluida(
        int $nrVinculo,
        string  $date
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrtipoprogocorrencia = self::NRTIPOPROGOCORRENCIA_SOLICITACAOCOMPENSARFALTA;

        $ocorrenciaSolicitacaoCompensacaoFalta = $this->getOcorrenciasDia(
            $nrOrg,
            $nrVinculo,
            $date,
            $nrtipoprogocorrencia
        );

        $jaFoiIncluidaSolicitacao = !empty($ocorrenciaSolicitacaoCompensacaoFalta);

        return $jaFoiIncluidaSolicitacao;
    }

    public function verificaSolicitacaoAbonoFaltaJaIncluida(
                $nrOrg,
                $nrVinculo,
        string  $date
    ) {
        $nrtipoprogocorrencia = self::NRTIPOPROGOCORRENCIA_SOLICITACAOABONARFALTA;

        $ocorrenciaAbonoCompensacaoFalta = $this->getOcorrenciasDia(
            $nrOrg,
            $nrVinculo,
            $date,
            $nrtipoprogocorrencia
        );

        $jaFoiIncluidaSolicitacao = !empty($ocorrenciaAbonoCompensacaoFalta);

        return $jaFoiIncluidaSolicitacao;
    }
    
    public function verificaOcorrenciaTrocaHorariosJaIncluida(
                $nrOrg,
                $nrVinculo,
        string  $date
    ) {
        $nrtipoprogocorrencia = self::NRTIPOPROGOCORRENCIA_TROCAHORARIOS;

        $ocorrenciaAbonoCompensacaoFalta = $this->getOcorrenciasDia(
            $nrOrg,
            $nrVinculo,
            $date,
            $nrtipoprogocorrencia
        );

        $jaFoiIncluidaSolicitacao = !empty($ocorrenciaAbonoCompensacaoFalta);

        return $jaFoiIncluidaSolicitacao;
    }

    public function solicitarCompensacaoHoras(
        int    $nrVinculo,
        string $date
    ): array {
        $status = $this->solicitaCompensacaoDia(
            $nrVinculo,
            $date,
            false
        );

        return $status;
    }

    public function solicitarHorasExtras(
        int     $nrVinculo,
        string  $observacao,
        ?string $file,
        string  $date,
        bool    $validaExistenciaCalculo = self::APROVAHE_VALIDA_EXISTENCIA_CALCULO,
        bool    $exigeAprovacao = self::APROVAHE_EXIGE_APROVACAO,
        $cdOperador = null
    ): array {
        $nrOrg              = (int) $this->environment->getNrOrg();
        $nrTipoOcorrencia   = self::NRTIPOPROGOCORRENCIA_SOLICITACAOHORASEXTRAS;
        $iddiassemana       = EmsUtil::getIdDiaSemana($date);
        $dsobservacao       = $observacao;

        // Verifica se é feriado
        $isDiaFeriado = $this->vinculoService->isDiaFeriadoVinculo($nrVinculo, $date);

        // Validações
        if ($isDiaFeriado) {

            $solicitacaoHoraExtraExists = $this->verifySolicitacaoHoraExtraExists(
                $nrVinculo, $date
            );

            if ($solicitacaoHoraExtraExists['exists']) {
                return ['success' => false, 'message' => $solicitacaoHoraExtraExists['message']];
            }

            // Remove ocorrência de abono de falta, caso exista
            $ocorrenciaAbonoFalta = $this->ocorrenciaRepository->getOcorrenciasPorTipoOcorrencia($nrVinculo, $nrOrg, self::NRTIPOPROGOCORRENCIA_SOLICITACAOABONARFALTA, $date);
            if (!empty($ocorrenciaAbonoFalta)) {
                $rowRemoveOcor = [
                    'NRPROGOCORRENCIA' => $ocorrenciaAbonoFalta[0]['NRPROGOCORRENCIA'],
                    'NRORG' => $nrOrg,
                    'NRVINCULOM' => $nrVinculo,
                ];

                $this->removeOcorrenciaFreq($rowRemoveOcor);
            }

            $tempoTrabalhado    = InstanceProvider::getMarcacaoServiceInstance()->calculateHorasTrabalhadas($nrVinculo, $date);
            $dsobservacao       = strtr(self::TEXTO_PADRAO_SOLICITACAO_HORAS_EXTRAS, array('@HORAS@' => $tempoTrabalhado)) . $dsobservacao;

            $idtipodiacalendario    = self::IDTIPODIACALENDARIO_FERIADO;
            $idtipohorario          = self::IDTIPOHORARIO_FOLGA;
        } else {

            /* Validacao da existencia previa do calculo de HE */

            if ($validaExistenciaCalculo) {
                $status = $this->verifyDiaHorasExtras(
                    $nrVinculo, $date
                );
    
                if (!$status['success']) {
                    return $status;
                }

                $horasExtras    = DateUtil::formataHoraDecimal($status['horasExtras']);
                $dsobservacao   = strtr(self::TEXTO_PADRAO_SOLICITACAO_HORAS_EXTRAS, array('@HORAS@' => $horasExtras)) . $dsobservacao;
            }

            $idtipohorario          = null;
            $idtipodiacalendario    = null;
        }


        $ocorrenciaParams = [
            'NRTIPOPROGOCORRENCIA'  => $nrTipoOcorrencia,
            'NRVINCULOM'            => $nrVinculo,
            'DTINICIOPROGOCOR'      => $date,
            'DTFIMPROGOCOR'         => $date,
            'NRORG'                 => $nrOrg,
            'IDDIASSEMANA'          => $iddiassemana,
            'DSOBSERVACAO'          => $dsobservacao,
            'IDTIPOHORARIO'         => $idtipohorario,
            'IDTIPODIACALENDARIO'   => $idtipodiacalendario,
            'FILE'                  => $file,
            'CDOPERADOR'            => $cdOperador
        ];

        if (empty($exigeAprovacao)) {
            $ocorrenciaParams['IDPROGOCORCONFIRMADA'] = self::IDPROGOCORCONFIRMADA_CONFIRMADA;
        }

        $this->setOcorrenciaFreq($ocorrenciaParams);

        return array('success' => true);
    }

    public function verifyPermission(
        string $cdOperador
    ): bool {
        $isDP = $this->operadorService->operadorIsDP($cdOperador);
        $isIdAprovOcorrencia = $this->paramOrgService->isIdAprovOcorrencia();

        // Verifica se o operador que esta a aprovar/rejeitar ocorrencias esta autorizado a realizar esse procedimento.
        // O operador estara apto se tiver perfil de acesso administrador (DP) ou o parametro da organizacao IDAPROVOCORRENCIA estiver marcado como 'S'
        if (!$isDP && !$isIdAprovOcorrencia) {
            throw new \Exception(Messages::OPER_NOT_AUTHORIZED);

            return false;
        }

        return true;
    }

    public function verificarPeriodo(
        int    $nrVinculo,
        string $date
    ) {
        $nrOrg = $this->environment->getNrOrg();

        $periodo = $this->periodoRepository->findPeriodoByDia(
            $nrOrg, $nrVinculo, $date
        );

        if (empty($periodo)) {
            throw new \Exception(Messages::PERIODO_PER_NOT_FOUND);
        }
    }

    /**
     *  A partir de parametros definidos em ApuracaoService, determina qual é o tipo de ocorrência de acordo com o tempo trabalhado. Em uma jornada com horas trabalhadas entre 04:00h e 06:00, o intervalo de
     * 15 minutos é obrigatório. Nesse caso, o tipo de intervalo é 26. Para os demais casos, é 20.
     *
     *  @param  int     $nrVinculo,
     *  @param  string  $dia
     *
     *  @return NRTIPOOCORRENCIA | O Nr. Tipo Ocorrência relativo ao tempo trabalhado
     *
     */
    public function getTipoOcorrenciaAlmocoPorJornada(
        int     $nrVinculo,
        string  $dia,
        string  $idtipohorario = null
    ): string {
        // Ocorrencia de almoço convencional
        $defaultReturn  = OcorrenciaService::NRTIPOPROGOCORRENCIA_ALMOCO;
        $nrOrg          = (int) $this->environment->getNrOrg();

        // Busca pelo IDTIPOHORARIO do dia em questão, caso não tenha sido informado
        if (empty($idtipohorario)) {
            $dadosHorarioVinculo = $this->marcacaoRepository->getDadosHorarioVinculo($nrVinculo, $nrOrg, $dia);
            if (!empty($dadosHorarioVinculo)) {
                $idtipohorario = $dadosHorarioVinculo['IDTIPOHORARIO'];
            }
        }

        // Verifica se o dia é folga. Caso o dia seja folga, é retornado o valor de NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG
        if (in_array($idtipohorario, OcorrenciaService::IDTIPOHORARIO_FOLGA_COMP)) {
            return OcorrenciaService::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG;
        }

        // Checa o tempo trabalhado
        $tempoTrabalhado = InstanceProvider::getMarcacaoServiceInstance()->calculateHorasTrabalhadas($nrVinculo, $dia);

        if (empty($tempoTrabalhado)) {
            return $defaultReturn;
        }

        $horasTrabalhadasTime = DateUtil::formataHoraString($tempoTrabalhado);

        $isTempoTrabalhoFaixaIntervalo15 =
            $horasTrabalhadasTime > ApuracaoService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MIN &&
            $horasTrabalhadasTime <= ApuracaoService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MAX;

        if ($isTempoTrabalhoFaixaIntervalo15 && $nrOrg === 20) {
            // Ocorrencia de intervalo não programado
            return OcorrenciaService::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG;
        }

        return OcorrenciaService::NRTIPOPROGOCORRENCIA_ALMOCO;
    }

    /**
     *  Rotina destinada à atualização das informações de tipo de programação de ocorrência e de horário projetado de intervalo de almoço associado à marcação
     *
     *  @param  int     $nrVinculo
     *  @param  string  $nrMarcacao
     *  @param  string  $dia
     *
     *  @return <UPDATE RESULT>
     *
     */
    public function updateOcorrenciaAlmoco(
        int     $nrVinculo,
        string  $nrMarcacao,
        string  $dia
    ) {
        $nrTipoOcorrencia           = $this->getTipoOcorrenciaAlmocoPorJornada($nrVinculo, $dia);
        $hrprojetarintervalonprog   = $this->getHrptojetainteravaloByTipoOcorrencia($nrVinculo, $dia, $nrTipoOcorrencia);
        $nrOrg                      = $this->environment->getNrorg();

        return $this->ocorrenciaRepository->updateOcorrenciaAlmoco(
            $nrOrg,
            $nrMarcacao,
            $nrTipoOcorrencia,
            $hrprojetarintervalonprog
        );
    }

    /**
     *  Rotina que identifica o horário a ser projetado a partir do tipo de ocorrência informado. Caso o tipo de ocorrência seja 26, o valor das horas projetadas é 0015. Para os demais valores, null
     * é retornado.
     *
     *  @param  string  $nrVinculo
     *  @param  string  $dia
     *  @param  string  $nrTipoOcorrencia
     *
     *  @return bool
     *
     */
    public function getHrptojetainteravaloByTipoOcorrencia(
                $nrVinculo,
        string  $dia,
        string  $nrTipoOcorrencia
    ) {
        switch ($nrTipoOcorrencia) {
            case self::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG:
                // Verifica se o dia atual é folga para o vínculo
                $tempoTrabalhadoTotal = InstanceProvider::getMarcacaoServiceInstance()->calculateHorasTrabalhadas($nrVinculo, $dia);
                $tempoTrabalhado = DateUtil::formataHoraString($tempoTrabalhadoTotal);

                // Fazer valer os valores definidos nos horários?
                if ($tempoTrabalhado > ApuracaoService::APUR_TEMPO_MIN_SEM_ALMOCO && $tempoTrabalhado <= ApuracaoService::APUR_TEMPO_MAXIMO_SEM_ALMOCO) {
                    return self::HRPROJETARINTERVALONPROG_INT_NAO_PROG;
                } else if ($tempoTrabalhado > ApuracaoService::APUR_TEMPO_MAXIMO_SEM_ALMOCO) {
                    return self::HRPROJETARINTERVALONPROG_ALMOCO;
                }

                return null;
                break;

            case self::NRTIPOPROGOCORRENCIA_ALMOCO;
                return null;
                break;

            default:
                return null;
        }
    }

    public function verifyCompensarDia(
        int   $nrVinculo,
        string $date,
        bool  $diaInteiro = true
    ): array {
        $status = [
            'success' => true,
            'message' => null
        ];

        $jaFoiSolicitadaCompensacao = $this->verificaSolicitacaoCompensacaoFaltaJaIncluida(
            $nrVinculo,
            $date
        );

        if ($jaFoiSolicitadaCompensacao) {
            $message = "O dia $date já possui solicitação de compensação!";

            $status['success'] = false;
            $status['message'] = $message;

            return $status;
        }

        $status = $this->apuracaoService->validacaoProcessoDependeDiasApurados(
            $nrVinculo, [$date], $diaInteiro, false
        );

        if (!$status['success']) {
            return $status;
        }

        $temResumo = $this->apuracaoService->verificaDiaTemResumo(
            $nrVinculo, $date
        );

        if (!$temResumo) {
            $message = "É precisa apurar o dia $date antes de solicitar compensação!";

            $status['success'] = false;
            $status['message'] = $message;

            return $status;
        }

        $saldoSuficiente = $this->verifySaldoSuficienteCompensar(
            $nrVinculo, $date
        );

        if (!$saldoSuficiente) {
            $message = "Você não possui horas suficientes para o dia $date!";

            $status['success'] = false;
            $status['message'] = $message;

            return $status;
        }

        return $status;
    }

    public function verifySaldoSuficienteCompensar(
        int    $nrVinculo,
        string $date
    ): bool {
        $apuracao = $this->apuracaoService->getApuracaoPorData(
            $nrVinculo, $date
        );

        $saldo = $apuracao['SALDO_BH_ATUAL'] ?? 0;

        $horarioBase = $this->vinculoService->getHorarioBase($nrVinculo, $date, VinculoService::GETHORARIOBASE_THROW_EXCEPTION);

        $horasTrabalhadas = $apuracao['HORASNORMAIS'];
        $horasATrabalhar = $horarioBase['QTHORASTRABALHO'];

        $saldoTime = Time::fromFloat($saldo);
        $horasTrabalhadasTime = Time::fromFloat($horasTrabalhadas);
        $horasATrabalharTime = Time::fromString($horasATrabalhar);

        $tempoRestanteTime = $horasATrabalharTime->sub($horasTrabalhadasTime);
        $tempoRestante = $tempoRestanteTime->toFloat();

        if ($tempoRestante <= 0) {
            return false;
        }

        $saldoSuficiente = $saldoTime->compare($tempoRestanteTime) >= 0;

        return $saldoSuficiente;
    }

    public function verifyDiaHorasExtras(
        int    $nrVinculo,
        string $date
    ): array {
        $status = [
            'success'       => true,
            'message'       => null,
            'horasExtras'   => null
        ];

        // Validação para plantonista. Para plantonista não precisa fazer valiudação
        // if ($this->vinculoService->isHorarioEscalaTurno($nrVinculo, $date, VinculoService::GETHORARIOBASE_NOT_THROW_EXCEPTION)) {

            // $solicitacaoHoraExtraExists = $this->verifySolicitacaoHoraExtraExists(
            //     $nrVinculo, $date
            // );

            // if ($solicitacaoHoraExtraExists['exists']) {
            //     $status['success'] = false;
            //     $status['message'] = $solicitacaoHoraExtraExists['message'];

            //     return $status;
            // }

            // // Retorna a diferença do sia apurado
            // $diffJornada = $this->apuracaoService->getDiferencaDia($nrVinculo, $date);
            // $status['horasExtras'] = !empty($diffJornada) ? $diffJornada->getHours() : null;
            // return $status;
        // }

        $temResumo = $this->apuracaoService->verificaDiaTemResumo(
            $nrVinculo, $date
        );

        if (!$temResumo) {
            $message = 'É preciso apurar o dia antes de solicitar horas extras!';
            $status['success'] = false;
            $status['message'] = $message;

            return $status;
        }

        $solicitacaoHoraExtraExists = $this->verifySolicitacaoHoraExtraExists(
            $nrVinculo, $date
        );

        if ($solicitacaoHoraExtraExists['exists']) {
            $status['success'] = false;
            $status['message'] = $solicitacaoHoraExtraExists['message'];

            return $status;
        }

        $horasExtras = $this->verifyPossuiHoraExtra(
            $nrVinculo, $date
        );

        if (!$horasExtras) {
            $message = 'Não há hora extra no dia selecionado!';
            $status['success']      = false;
            $status['message']      = $message;
        } else {
            $status['horasExtras']  = $horasExtras;
        }


        return $status;
    }

    public function verifySolicitacaoHoraExtraExists(
        int    $nrVinculo,
        string $date
    ): Array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrTipoOcorrencia = self::NRTIPOPROGOCORRENCIA_SOLICITACAOHORASEXTRAS;

        $status = [
            'exists'        => true,
            'message'       => 'O dia já possui solicitação de horas extras!',
        ];

        $ocorrenciasHoraExtraDoDia = $this->ocorrenciaRepository->getOcorrenciasDia(
            $nrOrg, $nrVinculo, $date, $nrTipoOcorrencia
        );

        if (count($ocorrenciasHoraExtraDoDia) === 0) {
            // return true;
            $status['exists'] = false;
            $status['message'] = null;
        }

        return $status;
    }

    public function verifyPossuiHoraExtra(
        int    $nrVinculo,
        string $date
    ): float {
        $apuracao = $this->apuracaoService->getApuracaoPorData(
            $nrVinculo, $date
        );

        $horasExtras = $apuracao['HORASEXTRAS'] ?? 0;

        return $horasExtras;
    }

    public function removeOcorrenciaByDateTime(
        int    $nrVinculo,
        string $date,
        string $time
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $ocorrencia = $this->getOcorrenciaByDateMarcacao(
            $nrOrg, $nrVinculo, $time, $date
        );

        if (!$ocorrencia) {
            return;
        }

        $nrOcorrencia = $ocorrencia['NRPROGOCORRENCIA'];

        $row = [
            'NRORG'            => $nrOrg,
            'NRPROGOCORRENCIA' => $nrOcorrencia,
            'NRVINCULOM'       => $nrVinculo,
        ];

        $this->removeOcorrenciaFreq($row);
    }

    public function getOcorrenciasAdmissao(
        int $nrTipoVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $ocorrencias = $this->ocorrenciaRepository->getOcorrenciasAdmissao($nrOrg, $nrTipoVinculo);

        return $ocorrencias;
    }

    public function getMotivo(
        int $nrMotivo
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $motivo = $this->ocorrenciaRepository->getMotivo(
            $nrOrg, $nrMotivo
        );

        return $motivo;
    }

    public function getEventosAtivos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetenc = date('01/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc,
        ];

        $eventos = $this->ocorrenciaRepository->getEventosAtivos($params);

        return $eventos;
    }

    public function getDiagnost() {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetenc = date('01/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao
        ];

        $eventos = $this->ocorrenciaRepository->getDiagnost($params);

        return $eventos;
    }

    public function getPessoaConselhoReg() {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtMesCompetenc = date('01/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc
        ];

        $eventos = $this->ocorrenciaRepository->getPessoaConselhoReg($params);

        return $eventos;
    }
    
    /**
     *  Rotina responsável por buscar os registros de HORÁRIO existentes (GPE_HORDIAM-GPE_HORDIAH). A query utilizada é uma cópia da utilizada pelo HCM
     * 
     *  @param      $nrPeriodo | Default: null
     * 
     *  @return     Array (Response)
     */
    public function getHoradiah(
        $nrPeriodo=null
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        
        if (empty($nrPeriodo)) {
            $dtMesCompetenc = $this->environment->getDtmescomptencDiaCorrente();
        } else {
            $periodo = $this->periodoRepository->getPeriodoDemonstrativo(
                $nrOrg, $nrPeriodo
            );
            
            if (empty($periodo)) {
                throw new \Exception(Messages::PERIODO_PER_NOT_FOUND);
            }
            
            $periodo = $periodo[0];
            
            $dtMesCompetenc = $periodo['DTFINALAPURACAO'];
        }
        

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_DTMESCOMPETENC'  => $dtMesCompetenc
        ];

        $eventos = $this->ocorrenciaRepository->getHoradiah($params);

        return $eventos;
    }
    
    /**
     * Rotina que gera a ocorrência de TROCA DE HORÁRIOS a partir das DATAS e do VÍNCULO passados
     *
     *  @param          $nrVinculo,
     *  @param  Array   $dates
     *
     */
    public function setTrocasHorario(
                $nrVinculo,
        array   $dates,
        string  $dsobservacao,
        string  $nrHordiam,
        string  $file
    ) {
        try {

            $diasTrocaJaInserida = array();

            $nrOrg = $this->environment->getNrorg();
            
            $dates = DateUtil::sortDateArray($dates);

            foreach ($dates as $date) {

                $trocaInseridaComSucesso = $this->setTrocaHorario(
                    $nrOrg,
                    $nrVinculo,
                    $date,
                    $dsobservacao,
                    $nrHordiam,
                    $file,
                    false
                );

                $paramsRespostaInsercao = $trocaInseridaComSucesso[$date];

                $trocaNaoInserida = $paramsRespostaInsercao === self::DEFAULT_NOTSAVED_RETURN;

                if ($trocaNaoInserida) {
                    throw new \Exception(Messages::OCOR_TROCA_HORARIO_NOT_SAVED);
                }

                $trocaJaInserida = $paramsRespostaInsercao === self::DEFAULT_ALREADYINCLUDED_RETURN;

                if ($trocaJaInserida) {
                    array_push($diasTrocaJaInserida, $date);

                    // break for loop
                    continue;
                }

                $trocaNaoSalva = $paramsRespostaInsercao !== self::DEFAULT_INCLUDED_RETURN;

                if ($trocaNaoSalva) {
                    throw new \Exception(Messages::OCOR_TROCA_HORARIO_NOT_SAVED);
                }

            }

            $trocasJaforamInseridas = !empty($diasTrocaJaInserida);

            if ($trocasJaforamInseridas) {

                $todasSolicitacoesForamIncluidas = $dates === $diasTrocaJaInserida;
                if ($todasSolicitacoesForamIncluidas) {
                    throw new \Exception(Messages::OCOR_TROCA_HORARIO_ALL_INCLUDED);
                }

                $messageParams = array('@DATES@' => implode(', ', $diasTrocaJaInserida));
                throw new \Exception(strtr(Messages::OCOR_TROCA_HORARIO_ALREADY_INCLUDED_SPECIFIED_DATES, $messageParams));
            }

            return true;


        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     *  Rotina responsável por incluir ocorrências do tipo TROCA DE HORÁRIOS
     * 
     *  @param      $nrOrg
     *  @param      $nrVinculo
     *  @param      $date
     *  @param      $dsobservacao
     *  @param      $file
     *  @param      $nrHordiam
     *  @param      $throwsExceptionWhenAlreadyIncluded | Default: true.
     * 
     *  @return     [$date => IN [self::DEFAULT_ALREADYINCLUDED_RETURN, self::DEFAULT_NOTSAVED_RETURN, self::DEFAULT_INCLUDED_RETURN]]
     * 
     */
    public function setTrocaHorario(
                $nrOrg,
                $nrVinculo,
        string  $date,
        string  $dsobservacao,
        string  $nrHordiam,
        string  $file,
        bool    $throwsExceptionWhenAlreadyIncluded = true
    ) {
        try {
            
            $defaultAlreadyIncludedReturn   = self::DEFAULT_ALREADYINCLUDED_RETURN;
            $defaultNotSavedReturn          = self::DEFAULT_NOTSAVED_RETURN;
            $defaultIncludedReturn          = self::DEFAULT_INCLUDED_RETURN;

            $jaFoiIncluidaTroca = $this->verificaOcorrenciaTrocaHorariosJaIncluida(
                $nrOrg,
                $nrVinculo,
                $date
            );

            if ($jaFoiIncluidaTroca) {

                if ($throwsExceptionWhenAlreadyIncluded) {
                    throw new \Exception(Messages::OCOR_TROCA_HORARIO_ALL_INCLUDED);
                }

                return [$date => $defaultAlreadyIncludedReturn];
            }

            $iddiassemana = self::IDDIASSEMANA_DEFAULT;

            $newOcorrencia = [
                'NRTIPOPROGOCORRENCIA'  => self::NRTIPOPROGOCORRENCIA_TROCAHORARIOS,
                'NRVINCULOM'            => $nrVinculo,
                'DTINICIOPROGOCOR'      => $date,
                'DTFIMPROGOCOR'         => $date,
                'NRORG'                 => $nrOrg,
                'IDDIASSEMANA'          => $iddiassemana,
                'DSOBSERVACAO'          => $dsobservacao,
                'FILE'                  => $file,
                'NRHORDIAM'             => $nrHordiam,
                '__is_new'              => true,
            ];

            try {
                
                $this->setOcorrenciaFreq(
                    $newOcorrencia
                );
                
                // TO DO: Alteracao da ocorrencia de almoco, considerando que um novo horario sera utilizado para o dia

            } catch (\Exception $e) {
                return [$date => $defaultNotSavedReturn];
            }


            return [$date => $defaultIncludedReturn];


        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     *  Rotina responsável por incluir ocorrências do tipo AUTORIZAR MARCACAO
     * 
     *  @param  int     $nrVinculo
     *  @param  string  $data
     *  @param  int     $nrMarcacao
     *  @param  int     $nrMotivoJust
     *  @param  string  $justificativa = null
     * 
     *  @return bool
     * 
     */
    public function setAprovacaoMarcacaoOcorrencia(
        int     $nrVinculo,
        string  $data,
        int     $nrMarcacao,
        int     $nrMotivoJust,
        ?string $justificativa = null,
        ?string $anexoJustificativa = null,
        $cdOperador = null
    ): bool {
        $newOcorrencia = [
            'NRTIPOPROGOCORRENCIA'  => self::NRTIPOPROGOCORRENCIA_AUTORIZARMARCACAO,
            'NRVINCULOM'            => $nrVinculo,
            'CDOPERADOR'            => $cdOperador,
            'DTINICIOPROGOCOR'      => $data,
            'DTFIMPROGOCOR'         => $data,
            'NRMARCACAOPONTO'       => $nrMarcacao,
            'NRORG'                 => $this->environment->getNrOrg(),
            'NRMOTIVOOCORFREQ'      => $nrMotivoJust,
            'DSOBSERVACAO'          => $justificativa,
            'FILE'                  => $anexoJustificativa,
            '__is_new'              => true,
        ];
            
        return $this->setOcorrenciaFreq(
            $newOcorrencia
        );
    }
    
    public static function getInstance() {
        $instanceManager = InstanceManager::getInstance();
        return $instanceManager->getService('OcorrenciaService');
    }

    private function getNrEstruturas(
        int $nrEstrutura
    ) {
       $estruturas = null;

        if ($nrEstrutura !== -1) {
            $nrEstruturas = $this->estruturaService->getTodasEstruturasInferiores(
                $nrEstrutura, true
            );

            $estruturas = ArrayUtil::toStringSQL($nrEstruturas);
        }

        return $estruturas;
    }

    public function getSituFrequencias() {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtAtual = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);

        $situacoes = $this->ocorrenciaRepository->getSituFrequencias($nrOrg, $dtAtual);

        return $situacoes;
    }
}
