<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class CCTController extends Crud {

    protected $CCTService;
    protected $dataSourceName = '';
    protected $dataSourceItemCCT = 'fpa_ItemCCT_get';
    protected $dataSourceBeneficioCCT = 'vab_BeneficioCCT_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_ATIVO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,\HCM\FPA\Services\CCTService $CCTService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->CCTService = $CCTService;
        parent::__construct($dataSourceManager);
    }

    public function getCCT (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $ativoFiltro = false;
        foreach ($filterData->getConditions() as $value) {
            if($value['columnName'] == "P_ATIVO")
                $ativoFiltro = true;    
            }
        if(!$ativoFiltro)
                $filterData->addCondition('P_ATIVO','=', 'T');
        $filterData->setDataSourceName("fpa_CCT_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
     public function setCCT(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_CCT";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCADASTROCCT', $row)) {
                    if ($row['NRCADASTROCCT'] === '') {
                        $row['NRCADASTROCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_CADASTROCCT', 1, 12, true); 
                    }
                } else {
                    $row['NRCADASTROCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_CADASTROCCT', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteCCT(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->CCTService->validarDeleteItemCCT($request->getDataSet()->getRows()[0]);
            $this->CCTService->validarDeleteBeneficioCCT($request->getDataSet()->getRows()[0]);
            $this->dataSourceName = "fpa_CCT";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function recadastroCCT(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->CCTService->recadastro($request->getDataSet()->getRows()[0]);
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Replicação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getItemCCT (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceItemCCT);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
	    foreach($rows as $key => &$row){
            if(isset($row['NROCUPACAOM'])){
                $row['NROCUPACAOM'] = explode(",", $row['NROCUPACAOM']);
                
                foreach($row['NROCUPACAOM'] as $chave => $value)
                    $row['NROCUPACAOM'][$chave] = intval($value);
                    
                $row['NMOCUPACAOH'] = explode(",", $row['NMOCUPACAOH']);
            }
            
            if(isset($row['NRESCALATRABM'])){
                $row['NRESCALATRABM'] = explode(",", $row['NRESCALATRABM']);
                
                foreach($row['NRESCALATRABM'] as $chave => $value)
                    $row['NRESCALATRABM'][$chave] = intval($value);
                    
                $row['NMESCALATRABH'] = explode(",", $row['NMESCALATRABH']);
            }
            if(isset($row['NRSITUFUNCM'])){
                $row['NRSITUFUNCM'] = explode(",", $row['NRSITUFUNCM']);
                
                foreach($row['NRSITUFUNCM'] as $chave => $value)
                    $row['NRSITUFUNCM'][$chave] = intval($value);
                    
                $row['NMSITUFUNCH'] = explode(",", $row['NMSITUFUNCH']);
            }
            if(isset($row['NREVENTOM'])){
                $row['NREVENTOM'] = explode(",", $row['NREVENTOM']);
                
                foreach($row['NREVENTOM'] as $chave => $value)
                    $row['NREVENTOM'][$chave] = intval($value);
                    
                $row['NMEVENTOH'] = explode(",", $row['NMEVENTOH']);
            }
            if(isset($row['NRTPDEMISSAO'])){
                $row['NRTPDEMISSAO'] = explode(",", $row['NRTPDEMISSAO']);
                
                foreach($row['NRTPDEMISSAO'] as $chave => $value)
                    $row['NRTPDEMISSAO'][$chave] = intval($value);
                    
                $row['NMTPDEMISSAO'] = explode(",", $row['NMTPDEMISSAO']);
            }
        }
        $response->addDataSet(new DataSet("fpa_ItemCCT_get", $rows));         
        
        
    }
    
    public function setItemCCT(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_ItemCCT";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRITCADASTROCCT', $row)) {
                    if ($row['NRITCADASTROCCT'] === '') {
                        $row['NRITCADASTROCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_ITCADASTROCCT', 1, 12, true); 
                    }
                } else {
                    $row['NRITCADASTROCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_ITCADASTROCCT', 1, 12, true); 
                }
                    if($row['NROCUPACAOM'] <> ""){
                    $row['NROCUPACAOM'] = implode(',',$row['NROCUPACAOM']);
                    }
                    if($row['NRESCALATRABM']<> ""){
                    $row['NRESCALATRABM'] = implode(',',$row['NRESCALATRABM']);
                    }
                    if($row['NRSITUFUNCM'] <> ""){
                    $row['NRSITUFUNCM'] = implode(',',$row['NRSITUFUNCM']);
                    }
                    if($row['NRTPDEMISSAO'] <> ""){
                    $row['NRTPDEMISSAO'] = implode(',',$row['NRTPDEMISSAO']);
                    }
                    if($row['NREVENTOM'] <> ""){
                    $row['NREVENTOM'] = implode(',',$row['NREVENTOM']);
                    }   
         
     }
            
            $row['NRFAIXATABFOLHA'] = isset($row['NRFAIXATABFOLHA'])?$row['NRFAIXATABFOLHA']:null;
            $row['NRVALORREFERENCIAM'] = isset($row['NRVALORREFERENCIAM'])?$row['NRVALORREFERENCIAM']:null;

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteItemCCT(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_ItemCCT";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getBeneficioCCT (DTO\Request\Filter $request, DTO\Response $response) {
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceBeneficioCCT);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){

            // Faixa Salarial Titular
            if(isset($row['NRTABFAIXASALTIT'])){
                $row['NRTABFAIXASALTIT'] = explode(", ", $row['NRTABFAIXASALTIT']);
                
                foreach($row['NRTABFAIXASALTIT'] as $chave => $value)
                    $row['NRTABFAIXASALTIT'][$chave] = intval($value);
                    
                $row['NMTABFAIXASALTIT'] = explode(", ", $row['NMTABFAIXASALTIT']);
            }    

            // Faixa Salarial Dependente
            if(isset($row['NRTABFAIXASALDEP'])){
                $row['NRTABFAIXASALDEP'] = explode(", ", $row['NRTABFAIXASALDEP']);
                
                foreach($row['NRTABFAIXASALDEP'] as $chave => $value)
                    $row['NRTABFAIXASALDEP'][$chave] = intval($value);
                    
                $row['NMTABFAIXASALDEP'] = explode(", ", $row['NMTABFAIXASALDEP']);
            }

            // Tipos vínculos Empregatícios
            if(isset($row['DSVINCULOEMPREG'])){
                $row['DSVINCULOEMPREG'] = explode(", ", $row['DSVINCULOEMPREG']);
                
                foreach($row['DSVINCULOEMPREG'] as $chave => $value)
                    $row['DSVINCULOEMPREG'][$chave] = intval($value);
                    
                $row['NMTIPOVINCULOH'] = explode(", ", $row['NMTIPOVINCULOH']);
            }        

            // Eventos para desconsiderarem o multiplicador
            if(isset($row['DSINCIDEVENTO'])){
                $row['DSINCIDEVENTO'] = explode(", ", $row['DSINCIDEVENTO']);
                
                foreach($row['DSINCIDEVENTO'] as $chave => $value)
                    $row['DSINCIDEVENTO'][$chave] = intval($value);
                    
                $row['NMEVENTOM_MULT_LIST'] = explode(", ", $row['NMEVENTOM_MULT_LIST']);
            }

            // Eventos para perder o benefício
            if(isset($row['DSINCIDEVENPERDER'])){
                $row['DSINCIDEVENPERDER'] = explode(", ", $row['DSINCIDEVENPERDER']);
                
                foreach($row['DSINCIDEVENPERDER'] as $chave => $value)
                    $row['DSINCIDEVENPERDER'][$chave] = intval($value);
                    
                $row['NMEVENTOM_PERD_LIST'] = explode(", ", $row['NMEVENTOM_PERD_LIST']);
            }

            // Situações funcionais para gerar ou não desconto e gerar ou não benefício
            if(isset($row['DSAFASTAMENTO1'])){
                $row['DSAFASTAMENTO1'] = explode(", ", $row['DSAFASTAMENTO1']);
                
                foreach($row['DSAFASTAMENTO1'] as $chave => $value)
                    $row['DSAFASTAMENTO1'][$chave] = intval($value);
                    
                $row['LIST_NMSITUFUNCM1'] = explode(", ", $row['LIST_NMSITUFUNCM1']);
            }

            // Situações funcionais para gerar ou não desconto e gerar ou não benefício
            if(isset($row['DSAFASTAMENTO2'])){
                $row['DSAFASTAMENTO2'] = explode(", ", $row['DSAFASTAMENTO2']);
                
                foreach($row['DSAFASTAMENTO2'] as $chave => $value)
                    $row['DSAFASTAMENTO2'][$chave] = intval($value);
                    
                $row['LIST_NMSITUFUNCM2'] = explode(", ", $row['LIST_NMSITUFUNCM2']);
            }

            // Situações funcionais para não gerar benefício
            if(isset($row['DSAFASTAMENTO3'])){
                $row['DSAFASTAMENTO3'] = explode(", ", $row['DSAFASTAMENTO3']);
                
                foreach($row['DSAFASTAMENTO3'] as $chave => $value)
                    $row['DSAFASTAMENTO3'][$chave] = intval($value);
                    
                $row['LIST_NMSITUFUNCM3'] = explode(", ", $row['LIST_NMSITUFUNCM3']);
            }

            // ID Gera Benefício somente para vínculos com mais de x dias trabalhados no mês
            if(isset($row['QTDIASMINMES'])) $row['QTDIASMINMES'] = strval($row['QTDIASMINMES']);
            
        }

        $response->addDataSet(new DataSet($this->dataSourceName, $rows));    
    }
    
    public function setBeneficioCCT(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "vab_BeneficioCCT";

            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $this->CCTService->verificarTpBeneficioJahCadastrado( $this->hcmParameters->getNrorgTrabalho(), $row['NRCADASTROCCT'], $row['NRTPBENEFICIO'], isset($row['NRBENEFICIOCCT']) && !empty($row['NRBENEFICIOCCT']) ? $row['NRBENEFICIOCCT'] : null );

                if (array_key_exists('NRBENEFICIOCCT', $row)) {
                    if ($row['NRBENEFICIOCCT'] === '') {
                        $row['NRORG'] = $this->hcmParameters->getNrorgPadrao();
                        $row['NRBENEFICIOCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'VAB_BENEFICIOCCT', 1, 12, true); 
                    }
                } else {
                    $row['NRBENEFICIOCCT'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'VAB_BENEFICIOCCT', 1, 12, true); 
                }
                
                
                $this->CCTService->validaCamposVaziosBeneficioCCT($row);
                
                // Faixa Salarial Titular
                if(isset($row['NRTABFAIXASALTIT']) && is_array($row['NRTABFAIXASALTIT']) )
                    $row['NRTABFAIXASALTIT'] = implode(", ", $row['NRTABFAIXASALTIT']);                        

                // Faixa Salarial Dependente
                if(isset($row['NRTABFAIXASALDEP']) && is_array($row['NRTABFAIXASALDEP']) )
                    $row['NRTABFAIXASALDEP'] = implode(", ", $row['NRTABFAIXASALDEP']);  

                // Tipos vínculos Empregatícios
                if(isset($row['DSVINCULOEMPREG']) && is_array($row['DSVINCULOEMPREG']) )
                    $row['DSVINCULOEMPREG'] = implode(", ", $row['DSVINCULOEMPREG']);  

                // Eventos para desconsiderarem o multiplicador
                if(isset($row['DSINCIDEVENTO']) && is_array($row['DSINCIDEVENTO']) )
                    $row['DSINCIDEVENTO'] = implode(", ", $row['DSINCIDEVENTO']);  

                // Eventos para perder o benefício
                if(isset($row['DSINCIDEVENPERDER']) && is_array($row['DSINCIDEVENPERDER']) )
                    $row['DSINCIDEVENPERDER'] = implode(", ", $row['DSINCIDEVENPERDER']);  

                // Situações funcionais para gerar ou não desconto e gerar ou não benefício
                if(isset($row['DSAFASTAMENTO1']) && is_array($row['DSAFASTAMENTO1']) )
                    $row['DSAFASTAMENTO1'] = implode(", ", $row['DSAFASTAMENTO1']);  

                // Situações funcionais para gerar ou não desconto e gerar ou não benefício
                if(isset($row['DSAFASTAMENTO2']) && is_array($row['DSAFASTAMENTO2']) )
                    $row['DSAFASTAMENTO2'] = implode(", ", $row['DSAFASTAMENTO2']);  

                // Situações funcionais para não gerar benefício
                if(isset($row['DSAFASTAMENTO3']) && is_array($row['DSAFASTAMENTO3']) )
                    $row['DSAFASTAMENTO3'] = implode(", ", $row['DSAFASTAMENTO3']);  
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
	public function deleteBeneficioCCT(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "vab_BeneficioCCT";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  

    }

    public function salarioEstaEntrePisoTetoCCT(DTO\Request\DataSet $request, DTO\Response $response){
        try { 
            foreach ($request->getDataSet()->getRows()[0] as &$row) {

                if(!empty($row['parametros'])){
                    $parametros = $row['parametros'];
                }
                if(!empty($row['movimentacaoCandidato'])){
                    $movimentacao = $row['movimentacaoCandidato'];

                }
                if(!empty($row['Pessoa'])){
                    $pessoa = $row['Pessoa'];
                    $pessoa['NRORG'] = array_key_exists('NRORG', $pessoa) ? $pessoa['NRORG'] : $this->hcmParameters->getNrorgTrabalho(); 
                }
            }
            
            if(isset($movimentacao['NRESTRUTSIND'])) {
                $salarioDentroCondicoesCCT = $this->CCTService->validaCondicoesCCT($this->hcmParameters->getNrorgTrabalho(), $movimentacao['NRESTRUTSIND'], $parametros);
                if(!$salarioDentroCondicoesCCT) {
                    $msgRetorno[] = 'Salario não esta de acordo com a cct';
                    $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('vinculoNãoSeEnquadraCCT', $msgRetorno));
                }  
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  

    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}