<?php

namespace HCM\FPA\ObrigacaoLegal\Rais;

use Teknisa\Model\Gestaopessoas\Entities\Base\FpaRaismagnet;
use Teknisa\Model\Gestaopessoas\Entities\Base\GpeGrauinstr;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;

use Teknisa\Model\Gestaopessoas\Repositories\GpeAltesitufunc;
use Teknisa\Model\Gestaopessoas\Repositories\Gpemovimentacao;
use Teknisa\Model\Gestaopessoas\Repositories\FpaCalculofolha;

use HCM\FPA\ObrigacaoLegal\Rais\Entities\Tipo02Base;
use HCM\FPA\ObrigacaoLegal\Rais\QuerysRais;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\Util\MsgCalculo;
use Geral\Util\NovoCodigo;



class Tipo02 extends Tipo02Base {

    private $ano;
    private $anoAnt;
    private $pessoa;
    private $pessoah;
    private $vinculo;
    private $vincRescAnoAnt;
    private $dtRescisaoVinc;
    private $estruturaSindical;
    private $nrformrais;
    private $nrestrutlegal;
    private $nrestrutgeren;
    private $movimentacao;
    private $dtmovtransfentr;
    private $dtmovtransfsaid;

    private $movSaidaRetorno;

    function __construct($hcmParameters,$entityManager,$vinculom, $ano, $cnpj, $grupoIncidencia, $nrformrais, $movSaidaRetorno = null) {
        try {
            $this->hcmParameters = $hcmParameters;
            $this->entityManager = $entityManager;
            $this->queryRais = new QuerysRais($this->entityManager,$this->hcmParameters);
            $this->_buscaDadosFormulario($nrformrais);
            $this->ano = $ano;
            $this->anoAnt = $ano - 1;
            $this->vincRescAnoAnt = false;
            $this->dtRescisaoVinc;
            $this->movSaidaRetorno = $movSaidaRetorno;
            $this->setDefault($cnpj);
            $this->setBasicInfo($vinculom);
            $this->setPersonInfo();
            $this->setProfissInfo($grupoIncidencia, $idProjetaRescisao);
            $this->setAfastInfo();
            $this->nrformrais = $nrformrais;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function setDefault($cnpj)
    {
        $this->tipoRegistro = '2';
        $this->prefixo = '00';
        $this->alvara = 2;
        $this->bancoDados = null;
        $this->quantMeses = null;
        $this->filiadoSindicato = '2';
        $this->infoEmpresa = null;
        $this->cnpj = $cnpj;
        $this->cnpjConAssocPri = null;
        $this->valorConAssoPri = null;
        $this->cnpjConAssocSeg = null;
        $this->valorConAssoSeg = null;
        $this->cnpjConSindiSeg = null;
        $this->valorConSindSeg = null;
        $this->cnpjConAssisSeg = null;
        $this->valorConAssiSeg = null;
        $this->cnpjConConfeSeg = null;
        $this->valorConConfSeg = null;
        /*Layout 2018 - Rafael Pickbrenner pck 26/01/2018 tarefa 117174 - 12/04/2019 186354*/
        $this->teletrabalho    = '2'; //Não
        $this->trabalhointermi = '2'; //Não
        /*Layout 2020 - 11/03/2021 tarefa 320433*/
        $this->matricula = null;
        $this->categoria = null;

        return true;
    }

    private function setBasicInfo($vinculo)
    {
        try {
            $pessoa = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findby(
                array(
                    'nrorg'=> $vinculo->getNrorg(),
                    'nrpessoa'=> $vinculo->getNrpessoa()
                )
            );

            if(!$pessoa){
                $params = array(
                    $vinculo->getNrvinculom(),
                    $vinculo->getNmvinculom()
                );
                throw new \Exception("Não existe cadastro de pessoa para o vínculo ".$vinculo->getNrvinculom()."-".$vinculo->getNmvinculom());
            }

            $pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->retornaPessoaAtual($vinculo->getNrorg(),$vinculo->getNrpessoa(),"31/12/".$this->ano);

            if(!$pessoah){
                throw new \Exception("Não foi possível encontrar histórico para a pessoa ".$pessoa->getNrpessoa()."-".$vinculo->getNmvinculom());
            }

            $this->pessoa = $pessoa;
            $this->pessoah = $pessoah;
            $this->vinculo = $vinculo;

            $this->setMovTransferencia();

            if ($this->vinculo->getDtrescisaovinc()) {
                $this->dtRescisaoVinc = DateUtil::getDataDeString($this->vinculo->getDtrescisaovinc());
                if ($this->dtRescisaoVinc->format('Y') == $this->anoAnt) {
                    $this->vincRescAnoAnt = true;
                    //$this->dtRescisaoVinc->addMonth(1);
                    $this->dtRescisaoVinc = DateUtil::adicionaIntervalo($this->dtRescisaoVinc, 1, DateUtil::MES);
                }
            }

            return true;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function setPersonInfo()
    {
        try {
            
            $nacionalidade = $this->entityManager->getRepository(Repositories::GPE_NACIONALIDADE)->findOneBy(
                array(
                'nrnacionalidade' => $this->pessoah->historico->getNrnacionalid(),    
                )
            );
            $ginstrucao = $this->entityManager->getRepository(Repositories::GPE_GRAUINSTR)->findOneBy(
                array(
                'nrgrauinstr' => $this->pessoah->historico->getNrgrauinstr(),    
                )
            );


            if ($this->pessoah->historico->getNrpispaseppes()) {
                $this->codigoPisPasep = $this->pessoah->historico->getNrpispaseppes();
            } else {
                //throw new \Exception("Não foi possível encontrar número de PIS para a pessoa ".$this->pessoah->getNrpessoa()."-".$this->pessoah->getNmpessoa());
                MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar número de PIS para a pessoa ".$this->pessoah->getNrpessoa()." - ".$this->pessoah->historico->getNmpessoa());
            }
            $this->nomeEmpregado = $this->pessoah->historico->getNmpessoa();
            $this->dataNascimento = $this->pessoah->historico->getDtnascpessoa();
            $this->anoChegPais = $this->pessoah->historico->getNranochegapais();
            $this->cpf = $this->pessoah->historico->getNrcpfpessoa();
            $this->ctpsNumero = $this->pessoah->historico->getNrctpspessoa();
            $this->ctpsSerie = $this->pessoah->historico->getNrseriectpspes();

            if($nacionalidade){
                $this->nacionalidade = $nacionalidade->getCdrais();
            }else{
                //throw new \Exception("A nacionalidade da pessoa ".$this->pessoah->getNrpessoa()."-".$this->pessoah->historico->getNmpessoa()." não foi encontrada");
                MsgCalculo::getInstance()->adicionaMensagem("A nacionalidade da pessoa ".$this->pessoah->getNrpessoa()." - ".$this->pessoah->historico->getNmpessoa()." não foi encontrada");
            }

            if($ginstrucao instanceof GpeGrauinstr){
                $this->grauInstrucao = $ginstrucao->getCodrais();
            }else{
                //throw new \Exception("A grau de instrução da pessoa ".$this->pessoah->getNrpessoa()."-".$this->pessoah->historico->getNmpessoa()." não foi encontrada");
                MsgCalculo::getInstance()->adicionaMensagem("A grau de instrução da pessoa ".$this->pessoah->getNrpessoa()." - ".$this->pessoah->historico->getNmpessoa()." não foi encontrada");
            }

            if ($this->pessoah->historico->getIdsexopessoa() == 'M') {
                $this->sexo = '1';
            } else {
                $this->sexo = '2';
            }

            switch ($this->pessoah->historico->getNrracapessoa()) {
                case(1): {
                    $this->raca = 1;
                    break;
                }
                case(2): {
                    $this->raca = 2;
                    break;
                }
                case(3): {
                    $this->raca = 4;
                    break;
                }
                case(4): {
                    $this->raca = 6;
                    break;
                }
                case(5): {
                    $this->raca = 8;
                    break;
                }
                default: {
                $this->raca = 9;
                break;
                }
            }

            switch ($this->pessoah->historico->getNrcondfispes()) {
                case(1): {
                    $this->deficiencia = 2;
                    $this->tipoDeficiencia = 0;
                    break;
                }
                case(2): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 1;
                    break;
                }
                case(3): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 2;
                    break;
                }
                case(4): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 3;
                    break;
                }
                case(5): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 4;
                    break;
                }
                case(6): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 5;
                    break;
                }
                case(7): {
                    $this->deficiencia = 1;
                    $this->tipoDeficiencia = 6;
                    break;
                }
                case(null): {
                    $this->deficiencia = 2;
                    $this->tipoDeficiencia = 0;
                    break;
                }
            }

            return true;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function setProfissInfo($grupoIncidencia, $idProjetaRescisao) {
        try {
            
            $this->matricula = is_null($this->vinculo->historico->getCdmatriculaesocial()) ? $this->vinculo->getNrvinculom() : $this->vinculo->historico->getCdmatriculaesocial();
            
            $ultdiano = "31/12/".$this->ano;
            $escalaTrabalho = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABH)->_retornaUltimoHistorico($this->vinculo->historico->getNrescalatrabm(),$this->vinculo->getNrorg(),$ultdiano);
            $ocupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->retornaUltimoHistorico($this->vinculo->historico->getNrCargo(),$this->vinculo->getNrorg(),$ultdiano);
            
            $estruturaTrabalho = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($this->vinculo->getNrorg(),$this->vinculo->historico->getNrestrutgeren(),$ultdiano);
            if(!$estruturaTrabalho){
                //throw new \Exception("Não foi possível encontrar a estrutura gerencial para o vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar a estrutura gerencial para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
            }else{
                $enderecoEmpresa = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findOneBy(
                    array(
                    'nrorg' => $this->vinculo->getNrorg(),
                    'nrparcnegocio' => $estruturaTrabalho->getNrparcnegocio()
                    )
                );
                if(!$enderecoEmpresa) {
                    //throw new \Exception("Não foi possível encontrar o endereço para a estrutura gerencial ".$estruturaTrabalho->getNrestruturam()."-".$estruturaTrabalho->getNmestruturah());
                    MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar o endereço para a estrutura gerencial ".$estruturaTrabalho->getNrestruturam()." - ".$estruturaTrabalho->getNmestruturah());
                }
            }
            if($this->vinculo->historico->getNrestrutsind()){
                $this->estruturaSindical = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($this->vinculo->getNrorg(),$this->vinculo->historico->getNrestrutsind(),$ultdiano);
                
                if(!$this->estruturaSindical && ($this->vinculo->getNrtipovinculom() == 1)){ 
                    throw new \Exception("Não foi possível encontrar a estrutura sindical para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
                }
            }

            $tipoAdmissao = $this->entityManager->getRepository(Repositories::FPA_TPADMISSAO)->findOneBy(
                array(
                'nrtpadmissao' => $this->vinculo->getNrtpadmissao()
                )
            );

            $vinculoEmpreg = $this->entityManager->getRepository(Repositories::GPE_VINCULOEMPREG)->findOneBy(
                array(
                'nrvinculoempreg' => $this->vinculo->getNrvinculoempreg()
                )
            );

            $tpDemissao = $this->entityManager->getRepository(Repositories::FPA_TPDEMISSAO)->findOneBy(
                array(
                'nrtpdemissao' => $this->vinculo->getNrtpdemissao()
                )
            );


            if ($this->dtmovtransfentr) {
                $this->tipoAdmissao = self::TPADMISSAO_TRANSF_S_ONUS;
            } else {
                if ($tipoAdmissao) {
                    $this->tipoAdmissao = $tipoAdmissao->getCodrais();
                } else {
                    //throw new \Exception("Não foi possível encontrar o tipo de admissão para o vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                    MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar o tipo de admissão para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
                }
            }

            if ($this->dtmovtransfentr) {
                $this->dataAdmissao = $this->dtmovtransfentr;
            } else {
                $this->dataAdmissao = $this->vinculo->getDtadmissaovinc();
            }

            if ($this->dtmovtransfsaid) {
                $this->dataDesligamento = $this->dtmovtransfsaid;
            } else {
                if (substr($this->vinculo->getDtrescisaovinc(), 6, 4) == $this->ano) {
                    $this->dataDesligamento = $this->vinculo->getDtrescisaovinc();
                } elseif (substr($this->vinculo->getDtrescisaovinc(), 6, 4) == $this->anoAnt) {
                    $this->dataDesligamento = $this->dtRescisaoVinc->format('d/m/Y');
                } else {
                    $this->dataDesligamento = '';
                }
            }

            //Verifica se deve projetar ou não a data de rescisão. PCK 14/03/2018
            if($idProjetaRescisao == 1) {
                if ($this->vinculo->getDtavisoprevio() != ''
                        &&  $this->vinculo->getDtrescisaovinc() != ''
						&& !$this->dtmovtransfsaid
                        &&  $this->vinculo->getNrtpdemissao() == self::DEMISSAO_SEM_JUSTA_CAUSA) {
                    $diasPro = 0;
                    if($this->vinculo->getDtavisoprevio() === $this->vinculo->getDtrescisaovinc()) {
                        $diasPro = 30;
                    }
                    $dtAdmis = clone($this->vinculo->getDtadmissaovinc());
                    $dtResci = clone($this->vinculo->getDtavisoprevio());
                    $anosTra = DateUtil::getQtdeAnosEntre($dtAdmis, $dtResci); //Anos de empresa / Trabalho
                    $diasPro = $diasPro + ($anosTra * 3);
                    $dtResci = DateUtil::adicionaIntervalo($dtResci, $diasPro, DateUtil::DIA);
                    if ((int)$dtResci->format('Y') <= (int)$this->ano) { //Se a projeção for em outro ano esta não realiza a projeção
                        $this->dataDesligamento = $dtResci->format('d/m/Y');
                    } else{
                        $this->dataDesligamento = null;					
                        $this->idRescisaoAnoSeguinte = true;
					}
					
                }
            }

            if ($vinculoEmpreg) {
                $this->vinculoEmpregat = $vinculoEmpreg->getCodrais();
                
                $categEsoc = $this->entityManager->getRepository(Repositories::GPE_CATEGTRABESOC)->findOneBy(array('nrcategesoc' => $vinculoEmpreg->getNrcategesoc()));
                if ($categEsoc) {
                    $this->categoria = $categEsoc->getCdesocial();
                } else {
                    $this->categoria = null;
                }
            }else{
                throw new \Exception("Não foi possível encontrar o vínculo empregatício para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
            }
            
            if($vinculoEmpreg->getNrvinculoempreg() == self::VINCULO_MENOR_APRENDIZ){
                $this->trabalhoparcial = '2'; //Não - Menor aprendiz não tem regime de trabalho parcial
                $dataInicio = "01/01/$this->ano";
                $dataFinal = "31/12/$this->ano";
                $info = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->getDiasSituacaoFuncional($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $dataInicio, $dataFinal, self::LICENCAO_MATERNIDADE);
                if($info && isset($info[0]['DIAS'])){
                    if($info[0]['DIAS'] > 0){
                        $this->aprendizgravida = '1'; //Sim
                    }else{
                        $this->aprendizgravida = '2'; //Nao
                    }
                }else{
                    $this->aprendizgravida = '2'; //Nao
                }
            }else{
                $this->aprendizgravida = '2'; //Nao
            }
            
            if(!$escalaTrabalho){
                //throw new \Exception("Não foi possível encontrar a escala de trabalho para o vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar a escala de trabalho para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
            } else {
                $this->horasSemanais = $escalaTrabalho->getQthrsemesctrabh();
            
                /*Layout 2018 - Rafael Pickbrenner pck 26/01/2018 tarefa 117174 - 12/04/2019 186354*/
                if($this->horasSemanais <= self::HR_TRABALHO_PARCIAL){
                    $this->trabalhoparcial = '1'; //Sim
                }else{
                    $this->trabalhoparcial = '2'; //Não
                }
            }
            
            if(!$ocupacao){
                throw new \Exception("Não foi possível encontrar a ocupação para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
            }
            $cbo = $this->entityManager->getRepository(Repositories::GPE_CBO)->findOneBy(
                array(/*'nrorg' => $ocupacao->getNrorg(),*/
                      'nrcbo' => $ocupacao->getNrcbo())
                );
            if (!is_null($cbo)) {
                $this->cbo = $cbo->getCdrais();
            }else{
                throw new \Exception("O CBO da ocupação ".$ocupacao->getNrocupacaom()." não foi encontrada.");
            }
            
            if ($this->dtmovtransfsaid) {
                $this->codDesligamento = self::CODDESLIG_TRANSF_S_ONUS;
            } else {
                if ((substr($this->vinculo->getDtrescisaovinc(), 6, 4) == $this->ano) || (substr($this->vinculo->getDtrescisaovinc(), 6, 4) == $this->anoAnt)) {
                    if ($tpDemissao) {
                        $this->codDesligamento = $tpDemissao->getCodrais();
                    } elseif (!is_null($this->vinculo->getDtrescisaovinc())) {
                        $params = array(
                            $this->vinculo->getNrvinculom(),
                            $this->vinculo->getNmvinculom(),
                        );
                        throw new \Exception("Não foi possível encontrar o tipo de demissão para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
                    }
                }
            }

            if(is_object($estruturaTrabalho)){
                if(is_object($enderecoEmpresa) && !is_null($enderecoEmpresa->getCdmunicipio())){
                    $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(
                            array(
                            "cdmunicipio" => $enderecoEmpresa->getCdmunicipio(),
                            "cdpais" => $enderecoEmpresa->getCdpais(),
                            "sgestado" => $enderecoEmpresa->getSgestado()
                                )
                            );
                    $this->municipioTrabalho = $municipio->getCdmunicibge();
                }else{
                    //new \Exception("Não foi possível encontrar o municipio para a estrutura ".$estruturaTrabalho->getNrestruturam()."-".$estruturaTrabalho->getNmestruturah());
                    MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar o municipio para a estrutura ".$estruturaTrabalho->getNrestruturam()." - ".$estruturaTrabalho->getNmestruturah());
                }
            }

            $this->setPaymInfo($grupoIncidencia);
            return true;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function setPaymInfo($grupoIncidencia) {
        try {
            $inicio = "01/01/{$this->ano}";
            $final = "31/12/{$this->ano}";

            $inicioAnt = "01/12/{$this->anoAnt}";
            $finalAnt = "31/12/{$this->anoAnt}";
            $remuneracao = $this->entityManager->getRepository(Repositories::GPE_ALTESALARIO)->retornaAlteSalarioAtual($this->vinculo->getNrorg(),$this->vinculo->getNrvinculom(),$final);

            if (!$remuneracao) {
                //new \Exception("Não foi possível encontrar alguma alteração salarial para o vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar alguma alteração salarial para o vínculo ".$this->vinculo->getNrvinculom()." - ".$this->vinculo->getNmvinculom());
            } else {
                $this->salarioContratu = round($remuneracao->getVrsalario(),2)*100;
                $this->tipoSalContratu = $this->entityManager->getRepository(Repositories::FPA_TPMODALIDSAL)->findOneBy(
                array('nrtpmodalidsal'=>$remuneracao->getNrtpmodalidsal())    
                );

                switch($remuneracao->getIdtpsalario()){
                    case(self::MOD_SALARIO_MENSAL):{
                        $this->tipoSalContratu = 1;
                        break;
                    }
                    case(self::MOD_SALARIO_QUINZENAL):{
                        $this->tipoSalContratu = 2;
                        break;
                    }
                    case(self::MOD_SALARIO_SEMANAL):{
                        $this->tipoSalContratu = 3;
                        break;
                    }
                    case(self::MOD_SALARIO_DIARIO):{
                        $this->tipoSalContratu = 4;
                        break;
                    }
                    case(self::MOD_SALARIO_HORA):{
                        $this->tipoSalContratu = 5;
                        break;
                    }
                    case(self::MOD_SALARIO_TAREFA):{
                        $this->tipoSalContratu = 6;
                        break;
                    }
                    default:{
                    $this->tipoSalContratu = 7;
                    break;
                    }
                }
            }

            if ($this->vincRescAnoAnt) {
                $val_totais = $this->queryRais->getValoresTotaisAnoAntRais($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $inicioAnt, $finalAnt, $grupoIncidencia);
            } else {
                if ($this->dtmovtransfentr) {
                    $dtInicioAno = DateUtil::getDataDeString($inicio);
                    $dtAdmissao = DateUtil::getDataDeString($this->dataAdmissao);
                    if ($dtInicioAno < $dtAdmissao) {
                        $inicio = $this->dataAdmissao->format('d/m/Y');
                    }
                    unset($dtInicioAno);
                    unset($dtAdmissao);
                }
                if ($this->dtmovtransfsaid) {
                    $dtFinalAno = DateUtil::getDataDeString($final);
                    $dtDesligamento = DateUtil::getDataDeString($this->dataDesligamento);
                    if ($dtFinalAno > $dtDesligamento) {
                        $final = $this->dataDesligamento == '' ? $final : $this->dataDesligamento->format('d/m/Y');
                    }
                    unset($dtFinalAno);
                    unset($dtDesligamento);
                }
                $val_mensais = $this->queryRais->getValoresMensaisRais($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $inicio, $final, $grupoIncidencia, $this->nrestrutlegal, $this->nrestrutgeren);
                
                if ($this->vinculo->getNrorg() == 20) {
                    $val_totais = $this->queryRais->getValoresTotaisRaisOrg20($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $inicio, $final, $grupoIncidencia, $this->vinculo->getDtrescisaovinc(), $this->nrestrutlegal, $this->nrestrutgeren);
                } else {
                    $val_totais = $this->queryRais->getValoresTotaisRais($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $inicio, $final, $grupoIncidencia, $this->vinculo->getDtrescisaovinc(), $this->nrestrutlegal, $this->nrestrutgeren);
                }
            }
            
            $this->remunJaneiro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['JANEIRO']:0 ), 2) * 100;
            $this->remunFevereiro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['FEVEREIRO']:0 ), 2) * 100;
            $this->remunMarco = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['MARCO']:0 ), 2) * 100;
            $this->remunAbril = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['ABRIL']:0 ), 2) * 100;
            $this->remunMaio = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['MAIO']:0 ), 2) * 100;
            $this->remunJunho = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['JUNHO']:0 ), 2) * 100;
            $this->remunJulho = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['JULHO']:0 ), 2) * 100;
            $this->remunAgosto = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['AGOSTO']:0 ), 2) * 100;
            $this->remunSetembro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['SETEMBRO']:0 ), 2) * 100;
            $this->remunOutubro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['OUTUBRO']:0 ), 2) * 100;
            $this->remunNovembro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['NOVEMBRO']:0 ), 2) * 100;
            $this->remunDezembro = round(( isset($val_mensais[self::GRP_ITEM_REMUNERACAO])?$val_mensais[self::GRP_ITEM_REMUNERACAO]['DEZEMBRO']:0 ), 2) * 100;

            $this->horaExtraJaneiro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['JANEIRO']:0 ), 0);
            $this->horaExtFevereiro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['FEVEREIRO']:0 ), 0);
            $this->horaExtraMarco = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['MARCO']:0 ), 0);
            $this->horaExtraAbril = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['ABRIL']:0 ), 0);
            $this->horaExtraMaio = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['MAIO']:0 ), 0);
            $this->horaExtraJunho = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['JUNHO']:0 ), 0);
            $this->horaExtraJulho = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['JULHO']:0 ), 0);
            $this->horaExtraAgosto = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['AGOSTO']:0 ), 0);
            $this->horaExtrSetembro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['SETEMBRO']:0 ), 0);
            $this->horaExtrOutubro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['OUTUBRO']:0 ), 0);
            $this->horaExtrNovembro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['NOVEMBRO']:0 ), 0);
            $this->horaExtrDezembro = round(( isset($val_mensais[self::GRP_ITEM_HORASEXTRAS])?$val_mensais[self::GRP_ITEM_HORASEXTRAS]['DEZEMBRO']:0 ), 0);
            
            /*$this->remunJaneiro = 0;
            $this->remunFevereiro = 0;
            $this->remunMarco = 0;
            $this->remunAbril = 0;
            $this->remunMaio = 0;
            $this->remunJunho = 0;
            $this->remunJulho = 0;
            $this->remunAgosto = 0;
            $this->remunSetembro = 0;
            $this->remunOutubro = 0;
            $this->remunNovembro = 0;
            $this->remunDezembro = 0;

            $this->horaExtraJaneiro = 0;
            $this->horaExtFevereiro = 0;
            $this->horaExtraMarco = 0;
            $this->horaExtraAbril = 0;
            $this->horaExtraMaio = 0;
            $this->horaExtraJunho = 0;
            $this->horaExtraJulho = 0;
            $this->horaExtraAgosto = 0;
            $this->horaExtrSetembro = 0;
            $this->horaExtrOutubro = 0;
            $this->horaExtrNovembro = 0;
            $this->horaExtrDezembro = 0;*/
            
            $aux = $val_totais;
            $val_totais = array();
            foreach($aux as $valor){
                $val_totais[$valor['NRITEM']] = $valor;
            };
            $this->remun13Adianta = array_key_exists(self::GRP_ITEM_13SALADIANT,$val_totais)?round($val_totais[self::GRP_ITEM_13SALADIANT]['VALOR_TOTAL'],2)*100:0;
            $this->mes13Adianta = array_key_exists(self::GRP_ITEM_13SALADIANT,$val_totais)?$val_totais[self::GRP_ITEM_13SALADIANT]['REFERENCIA']:0;
            $this->remun13Final = array_key_exists(self::GRP_ITEM_13SALFINAL,$val_totais)?(round($val_totais[self::GRP_ITEM_13SALFINAL]['VALOR_TOTAL'],2)*100):0;
            $this->mes13Final = array_key_exists(self::GRP_ITEM_13SALFINAL,$val_totais)?($val_totais[self::GRP_ITEM_13SALFINAL]['REFERENCIA']):0;

            $this->avisoPrevio = array_key_exists(self::GRP_ITEM_AVISOINDENI,$val_totais)?(round($val_totais[self::GRP_ITEM_AVISOINDENI]['VALOR_TOTAL'],2)*100):0;

            $this->feriasIndenizada = array_key_exists(self::GRP_ITEM_FERIASINDEN,$val_totais)?(round($val_totais[self::GRP_ITEM_FERIASINDEN]['VALOR_TOTAL'],2)*100):0;

            $this->valorDissidio = array_key_exists(self::GRP_ITEM_DISSIDIOCOL,$val_totais)?(round($val_totais[self::GRP_ITEM_DISSIDIOCOL]['VALOR_TOTAL'],2)*100):0;
            $this->quantMesesDissid = array_key_exists(self::GRP_ITEM_DISSIDIOCOL,$val_totais)?($val_totais[self::GRP_ITEM_DISSIDIOCOL]['REFERENCIA']):0;

            if (substr($this->vinculo->getDtrescisaovinc(), 6, 4) == $this->ano) {
                $this->gratificacoes = array_key_exists(self::GRP_ITEM_GRATIFICAC,$val_totais)?(round($val_totais[self::GRP_ITEM_GRATIFICAC]['VALOR_TOTAL'], 2) * 100):0;
                $this->mesesGratifica = array_key_exists(self::GRP_ITEM_GRATIFICAC,$val_totais)?($val_totais[self::GRP_ITEM_GRATIFICAC]['QUANTMESES']):0;
            }

            $this->multRescSemJust = array_key_exists(self::GRP_ITEM_MULTARESCSJ,$val_totais)?(round($val_totais[self::GRP_ITEM_MULTARESCSJ]['VALOR_TOTAL'],2)*100):0;
            
            
            
            if ((((isset($val_totais[self::GRP_ITEM_CONTRISINDI])?$val_totais[self::GRP_ITEM_CONTRISINDI]['VALOR_TOTAL']:0) > 0)|| ((isset($val_totais[self::GRP_ITEM_CONTRIASSOC])?$val_totais[self::GRP_ITEM_CONTRIASSOC]['VALOR_TOTAL']:0) > 0) ||
                 ((isset($val_totais[self::GRP_ITEM_CONTRIASSIS])?$val_totais[self::GRP_ITEM_CONTRIASSIS]['VALOR_TOTAL']:0) > 0)|| ((isset($val_totais[self::GRP_ITEM_CONTRICONFE])?$val_totais[self::GRP_ITEM_CONTRICONFE]['VALOR_TOTAL']:0) > 0)) &&
                (!$this->estruturaSindical || !$this->estruturaSindical->getCdcnpjestrut())) {

                $params = array(
                    $this->vinculo->getNrvinculom(),
                    $this->vinculo->getNmvinculom(),
                );
                
                throw new \Exception("Não foi possível encontrar a estrutura sindical para o vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
            } else {
                
                if(array_key_exists(self::GRP_ITEM_CONTRISINDI,$val_totais)){
                if ($val_totais[self::GRP_ITEM_CONTRISINDI]['VALOR_TOTAL'] > 0) {
                    $this->valorConSindSeg = round($val_totais[self::GRP_ITEM_CONTRISINDI]['VALOR_TOTAL'], 2) * 100;
                    $this->cnpjConSindiSeg = $this->estruturaSindical->getCdcnpjestrut();
                }}

                if(array_key_exists(self::GRP_ITEM_CONTRIASSOC,$val_totais)){
                if ($val_totais[self::GRP_ITEM_CONTRIASSOC]['VALOR_TOTAL'] > 0) {
                    $this->filiadoSindicato = 1;
                    $this->valorConAssoPri = round($val_totais[self::GRP_ITEM_CONTRIASSOC]['VALOR_TOTAL'], 2) * 100;
                    $this->cnpjConAssocPri = $this->estruturaSindical->getCdcnpjestrut();
                }}

                if(array_key_exists(self::GRP_ITEM_CONTRIASSIS,$val_totais)){
                if ($val_totais[self::GRP_ITEM_CONTRIASSIS]['VALOR_TOTAL'] > 0) {
                    $this->valorConAssiSeg = round($val_totais[self::GRP_ITEM_CONTRIASSIS]['VALOR_TOTAL'], 2) * 100;
                    $this->cnpjConAssisSeg = $this->estruturaSindical->getCdcnpjestrut();
                }}

                if(array_key_exists(self::GRP_ITEM_CONTRICONFE,$val_totais)){
                if ($val_totais[self::GRP_ITEM_CONTRICONFE]['VALOR_TOTAL'] > 0) {
                    $this->valorConConfSeg = round($val_totais[self::GRP_ITEM_CONTRICONFE]['VALOR_TOTAL'], 2) * 100;
                    $this->cnpjConConfeSeg = $this->estruturaSindical->getCdcnpjestrut();
                }}
            }

            return true;
        } catch (Exception $e) {
            if ($e->getCode() == Exception::EXCEPTION_ERRO)
                MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
            else
                throw $e;
        }
    }

    private function setAfastInfo()
    {
        try {
            $dataInicio = "01/01/$this->ano";
            $dataFinal = "31/12/$this->ano";

            if ($this->dtmovtransfentr) {
                $dtInicioAno = DateUtil::getDataDeString($dataInicio);
                $dtAdmissao = DateUtil::getDataDeString($this->dataAdmissao);
                if ($dtInicioAno < $dtAdmissao) {
                    $dataInicio = $this->dataAdmissao->format('d/m/Y');
                }
                unset($dtInicioAno);
                unset($dtAdmissao);
            }
            if ($this->dtmovtransfsaid) {
                $dtFinalAno = DateUtil::getDataDeString($dataFinal);
                $dtDesligamento = DateUtil::getDataDeString($this->dataDesligamento);
                if ($dtFinalAno > $dtDesligamento) {
                    $dataFinal = $this->dataDesligamento == '' ? $dataFinal : $this->dataDesligamento->format('d/m/Y');
                }
                unset($dtFinalAno);
                unset($dtDesligamento);
            }

            $info = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->getAfastamento($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $dataInicio, $dataFinal);
            $diasAfastamento = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->getTotalAfastamento($this->vinculo->getNrvinculom(), $this->vinculo->getNrorg(), $dataInicio, $dataFinal);

            if(($info)&&(isset($info[0]['DTINISITUFUNC']))){
                $this->motPriAfastam = $info[0]['CODRAIS'];
                $this->dataIniPriAfast = $info[0]['DTINISITUFUNC'];
                if ($info[0]['DTFIMSITUFUNC']) {
                    $this->dataFimPriAfast = $info[0]['DTFIMSITUFUNC'];
                } else {
                    $this->dataFimPriAfast = '';
                }
            }

            if(($info)&&(isset($info[1]['DTINISITUFUNC']))){
                $this->motSegAfastam = $info[1]['CODRAIS'];
                $this->dataIniSegAfast = $info[1]['DTINISITUFUNC'];
                if ($info[1]['DTFIMSITUFUNC']) {
                    $this->dataFimSegAfast = $info[1]['DTFIMSITUFUNC'];
                } else {
                    $this->dataFimSegAfast = '';
                }
            }

            if(($info)&&(isset($info[2]['DTINISITUFUNC']))){
                $this->motTerAfastam = $info[2]['CODRAIS'];
                $this->dataIniTerAfast = $info[2]['DTINISITUFUNC'];
                if ($info[2]['DTFIMSITUFUNC']) {
                    $this->dataFimTerAfast = $info[2]['DTFIMSITUFUNC'];
                } else {
                    $this->dataFimTerAfast = '';
                }
            }

            if(($diasAfastamento)&&(isset($diasAfastamento[0]['TEMPO']))){
                $this->quantDias = $diasAfastamento[0]['TEMPO'];
            }

            if(!$info){
                $altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoFuncional($this->vinculo->getNrorg(),$this->vinculo->getNrvinculom(),FPA::getUltimoDiaMes());
                if (is_object($altesitufunc)) {
                    if ($altesitufunc->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_NORMAL && $altesitufunc->getNrsitufuncm() != GpealtesitufuncEntity::SIT_FUNC_DEMITIDO) {
                        $dtInicioAno = DateUtil::getDataDeString($dataInicio);
                        $dtFinalAno = DateUtil::getDataDeString($dataFinal);
                        if ($dtInicioAno > $altesitufunc->getDtinisitufunc() && (!$altesitufunc->getDtfimsitufunc() || $dtFinalAno < $altesitufunc->getDtfimsitufunc())) {
                            $situfunch = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia($this->vinculo->getNrorg(), 0, $altesitufunc->getNrsitufuncm(), $dataFinal);
                            if (is_object($situfunch)) {
                                if ($situfunch->getCodrais()) {
                                    $this->motPriAfastam = $situfunch->getCodrais();
                                } else {
                                    MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar código RAIS para a situação funcional ".$situfunch->getNrsitufuncm()."-".$situfunch->getNmsitufunch()." do vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                                }
                            } else {
                                $this->motPriAfastam = '00';
                                MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar a situação funcional do vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                            }
                            $this->dataIniPriAfast = $dataInicio;
                            $this->dataFimPriAfast = $dataFinal;
                            $this->quantDias = 365;
                        }
                    }
                } else {
                    MsgCalculo::getInstance()->adicionaMensagem("Não foi possível encontrar a alteração da situação funcional do vínculo ".$this->vinculo->getNrvinculom()."-".$this->vinculo->getNmvinculom());
                }
            }

            return true;
        } catch (Exception $e) {
            if ($e->getCode() == Exception::EXCEPTION_ERRO)
                MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
            else
                throw $e;
        }

    }

    public function getNrvinculom(){
        return $this->vinculo->getNrvinculom();
    }

    private function salvarMagnetico($linha, $cnpj, $admissao, $tipoInsc, $vinculo, $pis)
    {
        $competencia = $this->hcmParameters->getDtCompetencia();

        if(!($admissao instanceof DateTime)){
            $admissao = DateUtil::getDataDeString($admissao);
        }

        if(!($competencia instanceof DateTime)){
            $competencia = DateUtil::getDataDeString($competencia);
        }

        $mag = new FpaRaismagnet();
        $mag->setNrraismagnet(NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RAISMAGNET', 1, 22, true));
        $mag->setMescompetencia($competencia);
        $mag->setDslinharegistro($linha . '*');
        $mag->setCdinscestrut($cnpj);
        $mag->setDtadmissaovinc($admissao);
        $mag->setIdtpinscestrutu($tipoInsc);
        $mag->setNrvinculom($vinculo);
        $mag->setNrpispaseppes($pis);
        $mag->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $mag->setIdativo('S');
        $mag->setIdtiporegistro($this->nrformrais);
        $mag->setDtinclusao(DateUtil::getDataAtual());

        $mag->persist();
    }

    private function _buscaDadosFormulario($nrformrais) {
        $formulario = $this->entityManager->getRepository(Repositories::FPA_FORMRAIS)->findOneby( 
        array(
        "nrorg" => $this->hcmParameters->getNrorgTrabalho(),
        "nrformrais" => $nrformrais
        )
        );
        if (is_object($formulario) === true) {
            if ($formulario->getDsordem() == 'ESTRUTURA') {
                $this->nrestrutlegal = $formulario->getNomeinicial();
                $this->nrestrutgeren = $formulario->getNomefinal();
            }
        } else {
            throw new \Exception("Formulário não encontrado, contate o administrador.");
        }
    }

    private function setMovTransferencia() {
        if (is_null($this->movSaidaRetorno) || $this->movSaidaRetorno == 'RETORNO') {
            $movtransfentr = current($this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacaoTransfEntrada(
                $this->vinculo->getNrorg(), $this->vinculo->getNrvinculom(), FPA::getTipoEstruturaLegalOrg(), $this->hcmParameters->getDtCompetencia(),$this->nrestrutlegal, '01/01/' . $this->ano, '31/12/' . $this->ano
            ));
            if (!empty($movtransfentr)) {
                $this->dtmovtransfentr = $movtransfentr->getDtinimoviment();
            }
        }
        if (is_null($this->movSaidaRetorno) || $this->movSaidaRetorno == 'SAIDA') {
            $movtransfsaid = current($this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacaoTransfSaida(
                $this->vinculo->getNrorg(), $this->vinculo->getNrvinculom(), FPA::getTipoEstruturaLegalOrg(), $dtcompetencia = $this->hcmParameters->getDtCompetencia(),$this->nrestrutlegal, '01/01/' . $this->ano, '31/12/' . $this->ano
            ));
            if (!empty($movtransfsaid)) {
                $this->dtmovtransfsaid = $movtransfsaid->getDtinimoviment();
            }
        }
    }
} 